from __future__ import unicode_literals

import collections
import functools

from . import base


class TestCase(base.TestCase):
    """
    A test case for making assertions about the headers in a response.
    
    """
    def assertHeadersInclude(self, response, names):
        """
        Asserts that the response contains the given header name(s)
        
        """
        self.assertKeysIn(names, response)
    
    def assertHeadersExclude(self, response, names):
        """
        Asserts that the response does not contain the given header name(s).
        
        """
        self.assertKeysNotIn(names, response)
    
    def assertHeadersExact(self, response, headers):
        """
        Asserts that the response contains the names and values specified
        by headers.
        
        """
        self.assertItemsExact(response, headers)
    
    def create_header_assertions(self, include=None, exclude=None, exact=None):
        """
        Returns a tuple of header assertion callbacks for the given arguments.
        
        """
        include = include or ()
        exclude = exclude or ()
        exact = (
            exact
                if isinstance(exact, collections.Mapping)
                else dict(exact or {}))
        return (
            ((functools.partial(self.assertHeadersInclude, names=include),)
                if include
                else ()) +
            ((functools.partial(self.assertHeadersExclude, names=exclude),)
                if exclude
                else ()) +
            ((functools.partial(self.assertHeadersExact, headers=exact),)
                if exact
                else ()))
    
    def create_response_assertions(self, headers_include=None,
            headers_exclude=None, headers_exact=None, **kwargs):
        """
        Adds header assertions to the response assertions.
        
        """
        return (
            self.create_header_assertions(
                include=headers_include,
                exclude=headers_exclude,
                exact=headers_exact) +
            super(TestCase, self).create_response_assertions(**kwargs))



from __future__ import unicode_literals

import collections

from . import base


__all__ = ("HttpMethodAllow", "HttpMethodDeny",)


class HttpMethodAllow(base.Allow):
    """
    Allow the request if the HTTP method name is allowed and implemented.
    
    """
    def http_method_allow_test(self):
        """
        A hook to control when HTTP methods are allowed.
        
        """
        return True
    
    def get_allow_handler(self):
        """
        Allow the request if the test passes, deferring
        to self.http_method_not_allowed.
        
        """
        return (
            self.http_method_allow_test() and
            getattr(self, self.request.method.lower(), None) or
            self.http_method_not_allowed)


class HttpMethodDeny(base.Deny):
    """
    Deny the request if the HTTP method name is not allowed or not implemented.
    
    """
    def get_http_method_names(self):
        """
        A hook to override the allowed HTTP method names.
        
        """
        return self.http_method_names
    
    def http_method_deny_test(self):
        """
        A hook to control when HTTP methods are denied.
        
        """
        method = self.request.method.lower()
        return (
            method in self.get_http_method_names() and
            isinstance(getattr(self, method, None), collections.Callable))
    
    def get_deny_handler(self):
        """
        Deny the request if the test fails, deferring to super().
        
        """
        return (
            not self.http_method_deny_test() and
            self.http_method_not_allowed or
            super(HttpMethodDeny, self).get_deny_handler())



from __future__ import unicode_literals

import collections
import functools

from . import base


class TestCase(base.TestCase):
    """
    A test case for making assertions about the context in a response.
    
    """
    def assertContextIncludes(self, response, names):
        """
        Asserts that the response's context contains the given name(s).
        
        """
        self.assertTrue(
            hasattr(response, "context") and response.context,
            "The response must have a non-empty context attribute.")
        self.assertKeysIn(names, response.context)
    
    def assertContextExcludes(self, response, names):
        """
        Asserts that the respone's context does not contain the given name(s).
        
        """
        if hasattr(response, "context") and response.context:
            self.assertKeysNotIn(names, response.context)
    
    def assertContextExact(self, response, context):
        """
        Asserts that the response's context contains the names and values
        specified by context.
        
        """
        self.assertTrue(
            hasattr(response, "context") and response.context,
            "The response must have a non-empty context attribute.")
        self.assertItemsExact(response.context, context)
    
    def create_context_assertions(self, include=None, exclude=None, exact=None):
        """
        Returns a tuple of context assertion callbacks for the given arguments.
        
        """
        include = include or ()
        exclude = exclude or ()
        exact = (
            exact
                if isinstance(exact, collections.Mapping)
                else dict(exact or {}))
        return (
            ((functools.partial(self.assertContextIncludes, names=include),)
                if include
                else ()) +
            ((functools.partial(self.assertContextExcludes, names=exclude),)
                if exclude
                else ()) +
            ((functools.partial(self.assertContextExact, context=exact),)
                if exact
                else ()))
    
    def create_response_assertions(self, context_includes=None,
            context_excludes=None, context_exact=None, **kwargs):
        """
        Add context assertions to the response assertions.
        
        """
        return (
            self.create_context_assertions(
                include=context_includes,
                exclude=context_excludes,
                exact=context_exact) +
            super(TestCase, self).create_response_assertions(**kwargs))



from __future__ import unicode_literals

import types

from django import http
from django.utils import six

from .. import base


__all__ = ("TestCase",)


class TestCase(base.TestCase):
    """
    A test case for dynamically generating and testing class-based views
    without using the URL resolution framework.
    
    Use the view() method to generate a view function from view class(es)
    and attributes.
    
    """
    # All HTTP verbs that should be automatically generated when
    # a string is provided in self.view()'s attrs.
    generate_http_verbs = (
        b"get", b"head", b"options", b"post", b"put", b"patch", b"delete",)
    
    # Utilities.
    def generate_http_verb(self, verb, content):
        """
        Generates an HTTP verb handler method that returns a 200 OK response
        with the specified content.
        
        """
        def method(self, request, *args, **kwargs):
            return http.HttpResponse(content)
        method.__name__ = verb
        return method
    
    def view(self, view_classes, **attrs):
        """
        Generate a view from the given view class(es) and attributes.
        
        The view_classes argument may specify a single class or an iterable of
        classes to inherit from.
        
        Be sure to specify classes that inherit from
        django.views.generic.base.View, or a class that is duck
        type equivalent.
        
        The attrs argument should be a dictionary of values that will be
        added to the class at creation time.
        
        For methods that implement HTTP verbs (get, post, put, etc.),
        if the attrs specify a string, a method that returns a simple
        response containing the string will be automatically generated.
        
        """
        # Normalize the view classes.
        view_classes = (
            (view_classes,)
                if isinstance(view_classes, types.TypeType)
                else view_classes
                    if isinstance(view_classes, tuple)
                    else tuple(view_classes))
        
        # Generate methods for HTTP verbs?
        for method in self.generate_http_verbs:
            content = attrs.get(method, None)
            if isinstance(content, six.string_types):
                attrs[method] = self.generate_http_verb(method, content)
        
        # Create the view.
        return type(
            b"".join(
                (b"Test",) + 
                tuple(view_class.__name__ for view_class in view_classes)),
            view_classes,
            attrs).as_view()



from __future__ import unicode_literals

import collections
import functools

from . import base


class TestCase(base.TestCase):
    """
    A test case for making assertions about the cookies in a response.
    
    """
    def assertCookiesInclude(self, response, names):
        """
        Asserts that the response contains the given cookie name(s).
        
        """
        self.assertTrue(
            hasattr(response, "cookies") and response.cookies,
            "The response must have a non-empty cookies attribute.")
        self.assertKeysIn(names, response.cookies)
    
    def assertCookiesExclude(self, response, names):
        """
        Asserts that the response does not contain the given cookie name(s).
        
        """
        if hasattr(response, "cookies") and response.cookies:
            self.assertKeysNotIn(names, response.cookies)
    
    def assertCookiesExact(self, response, cookies):
        """
        Asserts that the response contains the names and cookie values
        specified by cookies.
        
        """
        self.assertTrue(
            hasattr(response, "cookies") and response.cookies,
            "The response must have a non-empty cookies attribute.")
        self.assertItemsExact(response.cookies, cookies)
    
    def create_cookie_assertions(self, include=None, exclude=None, exact=None):
        """
        Returns a tuple of cookie assertion callbacks for the given arguments.
        
        """
        include = include or ()
        exclude = exclude or ()
        exact = (
            exact
                if isinstance(exact, collections.Mapping)
                else dict(exact or {}))
        return (
            ((functools.partial(self.assertCookiesInclude, names=include),)
                if include
                else ()) +
            ((functools.partial(self.assertCookiesExclude, names=exclude),)
                if exclude
                else ()) +
            ((functools.partial(self.assertCookiesExact, cookies=exact),)
                if exact
                else ()))
    
    def create_response_assertions(self, cookies_include=None,
            cookies_exclude=None, cookies_exact=None, **kwargs):
        """
        Adds cookie assertions to the response assertions.
        
        """
        return (
            self.create_cookie_assertions(
                include=cookies_include,
                exclude=cookies_exclude,
                exact=cookies_exact) +
            super(TestCase, self).create_response_assertions(**kwargs))



from __future__ import unicode_literals

from django import http

from daydreamer.views import generic
from daydreamer.views.behaviors import cache

from . import base


class CachePageTestCase(base.TestCase):
    """
    Tests for the CachePage view behavior.
    
    """
    view_classes = (cache.CachePage, generic.View,)
    
    def test_cache_page(self):
        """
        Check that the response is cached.
        
        """
        content = self.unique()
        calls = []
        def get(self, request, *args, **kwargs):
            calls.append(None)
            return http.HttpResponse(content)
        self.assertViewBehavior(
            {"get": get},
            repeat=2,
            status_code=200,
            content=content)
        self.assertEqual(len(calls), 1)
    
    def test_cache_page_disabled(self):
        """
        Check that the response is not cached when the behavior is disabled.
        
        """
        content = self.unique()
        calls = []
        def get(self, request, *args, **kwargs):
            calls.append(None)
            return http.HttpResponse(content)
        self.assertViewBehavior(
            {"cache_page": False, "get": get},
            repeat=2,
            status_code=200,
            content=content)
        self.assertEqual(len(calls), 2)
    
    def test_cache_page_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence.
        
        """
        self.assertViewBehavior(
            status_code=405)


class CacheControlTestCase(base.TestCase):
    """
    Tests for the CacheControl view behavior.
    
    """
    view_classes = (cache.CacheControl, generic.View,)
    
    def test_defaults(self):
        """
        Check that the defaults do not set any cache control headers on
        the response.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content},
            status_code=200,
            content=content,
            headers_exclude="Cache-Control")
    
    def test_public(self):
        """
        Check that the public cache control header is set on the resopnse.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"cache_control_public": True, "get": content},
            status_code=200,
            content=content,
            headers_exact={"Cache-Control": "public"})
    
    def test_private(self):
        """
        Check that the private cache control header is set on the resopnse.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"cache_control_public": False, "get": content},
            status_code=200,
            content=content,
            headers_exact={"Cache-Control": "private"})
    
    def test_no_cache(self):
        """
        Check that the no_cache cache control header is set on the resopnse.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"cache_control_no_cache": True, "get": content},
            status_code=200,
            content=content,
            headers_exact={"Cache-Control": "no-cache"})
    
    def test_no_transform(self):
        """
        Check that the no_transform cache control header is set on the resopnse.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"cache_control_no_transform": True, "get": content},
            status_code=200,
            content=content,
            headers_exact={"Cache-Control": "no-transform"})
    
    def test_must_revalidate(self):
        """
        Check that the must_revalidate cache control header is set on
        the resopnse.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"cache_control_must_revalidate": True, "get": content},
            status_code=200,
            content=content,
            headers_exact={"Cache-Control": "must-revalidate"})
    
    def test_proxy_revalidate(self):
        """
        Check that the proxy_revalidate cache control header is set on
        the response.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"cache_control_proxy_revalidate": True, "get": content},
            status_code=200,
            content=content,
            headers_exact={"Cache-Control": "proxy-revalidate"})
    
    def test_max_age(self):
        """
        Check that the max_age cache control header is set on the resopnse.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"cache_control_max_age": 1, "get": content},
            status_code=200,
            content=content,
            headers_exact={"Cache-Control": "max-age=1"})
    
    def test_s_maxage(self):
        """
        Check that the s_maxage cache control header is set on the response.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"cache_control_s_maxage": 1, "get": content},
            status_code=200,
            content=content,
            headers_exact={"Cache-Control": "s-maxage=1"})
    
    def test_disabled(self):
        """
        Check that the behavior is disabled when cache_control is falsy.
        
        """
        content = self.unique()
        self.assertViewBehavior({
                "cache_control": False,
                "cache_control_public": True,
                "get": content},
            status_code=200,
            content=content,
            headers_exclude="Cache-Control")
    
    def test_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence and
        that no cache control headers are set on the response.
        
        """
        self.assertViewBehavior(
            {"cache_control_public": True},
            status_code=405,
            headers_exclude="Cache-Control")


class NeverCacheTestCase(base.TestCase):
    """
    Test for the NeverCache view behavior.
    
    """
    view_classes = (cache.NeverCache, generic.View,)
    
    def test_defaults(self):
        """
        Check that the defaults set the never cache control headers on
        the response.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content},
            status_code=200,
            content=content,
            headers_exact={"Cache-Control": "max-age=0"})
    
    def test_disabled(self):
        """
        Check that the behavior is disabled when never_cache is falsy.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"never_cache": False, "get": content},
            status_code=200,
            content=content,
            headers_exclude="Cache-Control")
    
    def test_precedence(self):
        """
        Check that the defualt HTTP method name protection takes precedence and
        that no cache control headers are set on the response.
        
        """
        self.assertViewBehavior(
            status_code=405,
            headers_exclude="Cache-Control")



from __future__ import unicode_literals

from django.views import generic

from daydreamer.views import core


__all__ = ("View", "TemplateView", "RedirectView",)


class View(core.HttpMethodDeny, core.HttpMethodAllow):
    """
    A replacement for django.views.generic.View. Implements a dispatch system
    with improved tools for denying or allowing a request to be processed.
    
    """
    pass


class TemplateView(generic.TemplateView, View):
    """
    Extends Django's TemplateView class with features from
    daydreamer.views.generic.View.
    
    """
    pass


class RedirectView(generic.RedirectView, View):
    """
    Extends Django's RedirectView class with features from
    daydreamer.views.generic.View.
    
    """
    pass



from __future__ import unicode_literals

from daydreamer.tests.views.core import http


class TestCase(http.TestCase):
    """
    Common utilities for testing vary view behaviors.
    
    """
    pass



from __future__ import unicode_literals

import collections
import functools

from . import base


class TestCase(base.TestCase):
    """
    A test case for making assertions about attributes on a request.
    
    """
    def assertRequestIncludes(self, request, names):
        """
        Asserts that the request contains the given attribute name(s).
        
        """
        self.assertAttributesIn(names, request)
    
    def assertRequestExcludes(self, request, names):
        """
        Asserts that the request does not contain the given attribute name(s).
        
        """
        self.assertAttributesNotIn(names, request)
    
    def assertRequestExact(self, request, attrs):
        """
        Asserts that the request contains the attribute names and values
        specified by attrs.
        
        """
        self.assertAttributesExact(request, attrs)
    
    def create_request_attribute_assertions(self, include=None, exclude=None,
            exact=None):
        """
        Returns a tuple of request assertion callbacks for the given arguments.
        
        """
        include = include or ()
        exclude = exclude or ()
        exact = (
            exact
                if isinstance(exact, collections.Mapping)
                else dict(exact or {}))
        return (
            ((functools.partial(self.assertRequestIncludes, names=include),)
                if include
                else ()) +
            ((functools.partial(self.assertRequestExcludes, names=exclude),)
                if exclude
                else ()) +
            ((functools.partial(self.assertRequestExact, attrs=exact),)
                if exact
                else ()))
    
    def create_request_assertions(self, request_includes=None,
            request_excludes=None, request_exact=None, **kwargs):
        """
        Adds request attribute assertions to the request assertions.
        
        """
        return (
            self.create_request_attribute_assertions(
                include=request_includes,
                exclude=request_excludes,
                exact=request_exact) +
            super(TestCase, self).create_request_assertions(**kwargs))



from __future__ import unicode_literals

from django import http

from daydreamer.core import urlresolvers
from daydreamer.views import core

from . import base


class CoreTestCase(base.TestCase):
    """
    Tests for the Core view base class.
    
    """
    view_classes = core.Core
    
    def test_attachment(self):
        """
        Check that the attachment method behaves correctly.
        
        """
        content = self.unique()
        content_type = "text/plain"
        filename = "{unique:s}.txt".format(unique=self.unique())
        def get(self, request, *args, **kwargs):
            return self.attachment(content, content_type, filename)
        self.assertViewBehavior(
            {"get": get},
            status_code=200,
            content=content,
            headers_exact={
                "Content-Type": content_type,
                "Content-Disposition":
                    "attachment; filename=\"{filename:s}\"".format(
                        filename=filename)})
    
    def test_redirect(self):
        """
        Check that the redirect method behaves correctly.
        
        """
        def get(self, request, *args, **kwargs):
            return self.redirect("admin:index")
        self.assertViewBehavior(
            {"get": get},
            redirect_url=urlresolvers.reverse("admin:index"))
    
    def test_gone(self):
        """
        Check that the gone method gets a 410 resopnse.
        
        """
        def get(self, request, *args, **kwargs):
            return self.gone()
        self.assertViewBehavior(
            {"get": get},
            status_code=410)
    
    def test_not_found(self):
        """
        Check that the not found method gets a 404 response.
        
        """
        def get(self, request, *args, **kwargs):
            return self.not_found()
        self.assertViewBehavior(
            {"get": get},
            status_code=404)
    
    def test_permission_denied(self):
        """
        Check that the permission denied method gets a 403 response.
        
        """
        def get(self, request, *args, **kwargs):
            return self.permission_denied()
        self.assertViewBehavior(
            {"get": get},
            status_code=403)
    
    def test_suspicious_operation(self):
        """
        Check that the suspicious operation method gets a 400 response.
        
        """
        def get(self, request, *args, **kwargs):
            return self.suspicious_operation()
        self.assertViewBehavior(
            {"get": get},
            status_code=400)


class NullTestCase(base.TestCase):
    """
    Tests for the Null view base class.
    
    """
    view_classes = core.Null
    
    def test_get_not_allowed(self):
        """
        Check that GET requests are not allowed.
        
        """
        self.assertViewBehavior(
            {"get": self.unique()},
            status_code=405)
    
    def test_head_not_allowed(self):
        """
        Check that HEAD requests are not allowed.
        
        """
        self.assertViewBehavior(
            {"head": self.unique()},
            method="head",
            status_code=405)
    
    def test_options_not_allowed(self):
        """
        Check that OPTIONS requests are not allowed.
        
        """
        self.assertViewBehavior(
            {"options": self.unique()},
            method="options",
            status_code=405)
    
    def test_post_not_allowed(self):
        """
        Check that POST requests are not allowed.
        
        """
        self.assertViewBehavior(
            {"post": self.unique()},
            method="post",
            status_code=405)
    
    def test_put_not_allowed(self):
        """
        Check that PUT requests are not allowed.
        
        """
        self.assertViewBehavior(
            {"put": self.unique()},
            method="put",
            status_code=405)
    
    def test_patch_not_allowed(self):
        """
        Check that PATCH requests are not allowed.
        
        """
        self.assertViewBehavior(
            {"patch": self.unique()},
            method="patch",
            status_code=405)
    
    def test_delete_not_allowed(self):
        """
        Check that DELETE requests are not allowed.
        
        """
        self.assertViewBehavior(
            {"delete": self.unique()},
            method="delete",
            status_code=405)


class DenyTestCase(NullTestCase):
    """
    Tests for the Deny view base class.
    
    """
    view_classes = core.Deny
    
    def test_deny_handler(self):
        """
        Check that a custom deny handler takes precedence.
        
        """
        content = self.unique()
        def custom_deny(self, request, *args, **kwargs):
            return http.HttpResponse(content)
        def get_deny_handler(self):
            return self.custom_deny
        self.assertViewBehavior({
                "custom_deny": custom_deny,
                "get_deny_handler": get_deny_handler},
            status_code=200,
            content=content)


class AllowTestCase(NullTestCase):
    """
    Tests for the Allow view base class.
    
    """
    view_classes = core.Allow
    
    def test_allow_handler(self):
        """
        Check that a custom allow handler takes precedence.
        
        """
        content = self.unique()
        def custom_allow(self, request, *args, **kwargs):
            return http.HttpResponse(content)
        def get_allow_handler(self):
            return self.custom_allow
        self.assertViewBehavior({
                "custom_allow": custom_allow,
                "get_allow_handler": get_allow_handler},
            status_code=200,
            content=content)



from __future__ import unicode_literals

import collections
import logging
import sys

from django import http
from django.conf import settings
from django.core import exceptions, signals, urlresolvers
from django.core.handlers import base
from django.utils import encoding, six
from django.views import debug

from daydreamer.core import lang


__all__ = ("Handler",)


logger = logging.getLogger("django.request")


class Handler(base.BaseHandler):
    """
    Provides a refactored get_response() method with a finer grain of control
    over response generation through object-oriented hooks.
    
    """
    def get_resolver(self, request):
        """
        Returns a django.core.urlresolvers.RegexURLResolver for the request's
        urlconf, falling back to the global urlconf fom settings.
        
        """
        urlconf = getattr(request, "urlconf", settings.ROOT_URLCONF)
        urlresolvers.set_urlconf(urlconf)
        return urlresolvers.RegexURLResolver(r'^/', urlconf)
    
    def get_resolver_match(self, request, resolver):
        """
        Resolves the view using the request and the URL resolver. Returns a
        triple containing the view, its args and its kwargs.
        
        """
        return resolver.resolve(request.path_info)
    
    def resolve_view(self, request, resolver):
        """
        If missing, caches the result of get_resolver_match() on the request.
        Returns the resolver match data. See get_resolver_match() for details.
        
        """
        if (not hasattr(request, "resolver_match") or
            request.resolver_match is None):
            request.resolver_match = self.get_resolver_match(request, resolver)
        return request.resolver_match
    
    def apply_request_middleware(self, request, resolver):
        """
        Apply request middleware in order, returning the first encountered
        response, falling back to None.
        
        """
        return lang.any((
                method(request)
                for method in self._request_middleware),
            None)
    
    def apply_view_middleware(self, request, resolver):
        """
        Resolve the view and apply view middleware in order, returning
        the first encountered response, falling back to None.
        
        """
        match = self.resolve_view(request, resolver)
        return lang.any((
                method(request, *match)
                for method in self._view_middleware),
            None)
    
    def apply_exception_middleware(self, request, resolver, exception):
        """
        Apply exception middleware in order, returning the first encountered
        response, falling back to None.
        
        """
        return lang.any((
                method(request, exception)
                for method in self._exception_middleware),
            None)
    
    def apply_view(self, request, resolver):
        """
        Resolve the view, make it atomic and apply it. If an exception occurs,
        apply the exception middleware.
        
        """
        view, view_args, view_kwargs = self.resolve_view(request, resolver)
        try:
            return self.make_view_atomic(view)(
                request, *view_args, **view_kwargs)
        except Exception as exception:
            return (
                self.apply_exception_middleware(
                    request, resolver, exception) or
                six.reraise(*sys.exc_info()))
    
    def validate_response(self, request, resolver, response):
        """
        If the response is falsy, raise a ValueError complaining about the view
        not returning a response.
        
        """
        if not response:
            view, = self.resolve_view(request, resolver)[:1]
            raise ValueError(
                "The view {module:s}.{name:s} didn't return an "
                "HttpResponse object.".format(
                    module=view.__module__,
                    name=view.__name__
                        if isinstance(view, types.FunctionType)
                        else view.__class__.__name__ + ".__call__"))
        return response
    
    def apply_template_response_middleware(self, request, resolver, response):
        """
        If the response has a render() method, transform the response by
        applying template response middleware in order, finally rendering
        the response.
        
        """
        if isinstance(getattr(response, "render", None), collections.Callable):
            for method in self._template_response_middleware:
                response = method(request, response)
            response = response.render()
        return response
    
    def apply_response_middleware(self, request, resolver, response):
        """
        Transform the response by applying reponse middleware in order.
        
        """
        for method in self._response_middleware:
            response = method(request, response)
        return response
    
    def apply_response_fixes(self, request, resolver, response):
        """
        Apply response fixes.
        
        Calls the base implementation. This hook is provided so that
        consistent arguments are passed to the various apply_* methods.
        
        """
        return super(Handler, self).apply_response_fixes(request, response)
    
    def handle_not_found(self, request, resolver, exception):
        """
        Logs a 404 warning and invokes either the debug handler or the
        resolver's 404 handler.
        
        """
        logger.warning(
            "Not Found: {path:s}".format(path=request.path),
            extra={"status_code": 404, "request": request})
        if settings.DEBUG:
            return debug.technical_404_response(request, exception)
        callback, kwargs = resolver.resolve404()
        return callback(request, **kwargs)
    
    def handle_permission_denied(self, request, resolver, exception):
        """
        Logs a 403 warning and invokes the resolver's 403 handler.
        
        """
        logger.warning(
            "Forbidden (Permission denied): {path:s}".format(
                path=request.path),
            extra={"status_code": 403, "request": request})
        callback, kwargs = resolver.resolve403()
        return callback(request, **kwargs)
    
    def handle_suspicious_operation(self, request, resolver, exception):
        """
        Logs a 400 security error and invokes the resolver's 400 handler.
        
        """
        logging.getLogger(
            ".".join(
                ("django.security",
                exception.__class__.__name__,))).error(
                    encoding.force_text(exception))
        callback, kwargs = resolver.resolve400()
        return callback(request, **kwargs)
    
    def handle_uncaught_exception(self, request, resolver, exc_info):
        """
        Signals the uncaught exception and invokes the
        super()'s implementation.
        
        """
        signals.got_request_exception.send(
            sender=self.__class__, request=request)
        return super(Handler, self).handle_uncaught_exception(
            request, resolver, exc_info)
    
    def view_response(self, request, resolver):
        """
        Generates an initial response from either the request middleware,
        the view middleware or the view, in that order.
        
        """
        return (
            self.apply_request_middleware(request, resolver) or
            self.apply_view_middleware(request, resolver) or
            self.apply_view(request, resolver))
    
    def render_response(self, request, resolver):
        """
        Validates the view response and applies the template response
        middleware, which finishes rendering of the response.
        
        """
        return self.apply_template_response_middleware(
            request, resolver, self.validate_response(
                request, resolver, self.view_response(
                    request, resolver)))
    
    def generate_response(self, request, resolver):
        """
        Renders an initial response, falling back to exception handler
        responses for particular exceptions.
        
        Subclasses wishing to handle more exception types should override this
        method to catch any additional exceptions raised by super().
        
        """
        try:
            return self.render_response(request, resolver)
        except http.Http404 as exception:
            return self.handle_not_found(
                request, resolver, exception)
        except exceptions.PermissionDenied as exception:
            return self.handle_permission_denied(
                request, resolver, exception)
        except exceptions.SuspiciousOperation as exception:
            return self.handle_suspicious_operation(
                request, resolver, exception)
    
    def process_response(self, request, resolver, response):
        """
        Processes the response with response middleware and response fixes,
        returning the finalized response.
        
        """
        return self.apply_response_fixes(
            request, resolver, self.apply_response_middleware(
                request, resolver, response))
    
    def get_response(self, request):
        """
        Overrides the base implementation with object-oriented hooks.
        
        Adapted from django.core.handlers.base.BaseHandler.
        
        """
        resolver = self.get_resolver(request)
        try:
            return self.process_response(
                request, resolver, self.generate_response(
                    request, resolver))
        except SystemExit:
            six.reraise(*sys.exc_info())
        except:
            return self.handle_uncaught_exception(
                request, resolver, sys.exc_info())



from __future__ import unicode_literals

from daydreamer.views.core import http

from . import base


class HttpMethodAllowTestCase(base.TestCase):
    """
    Tests for the HttpMethodAllow view base class.
    
    """
    view_classes = http.HttpMethodAllow
    
    def test_included_present_allowed(self):
        """
        Check that an included HTTP method name's handler is dispatched when
        the handler is present.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content, "http_method_names": ("get",)},
            status_code=200,
            content=content)
    
    def test_included_missing_denied(self):
        """
        Check that an included HTTP method name is denied when its handler
        is missing.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"http_method_names": ("get",)},
            status_code=405)
    
    def test_excluded_present_allowed(self):
        """
        Check that an excluded HTTP method name's handler is dispatched when
        the handler is present.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content, "http_method_names": ()},
            status_code=200,
            content=content)
    
    def test_excluded_present_denied(self):
        """
        Check that an excluded HTTP method name is denied when its handler is
        missing.
        
        """
        self.assertViewBehavior(
            {"http_method_names": ()},
            status_code=405)


class HttpMethodDenyTestCase(base.TestCase):
    """
    Tests for the HttpMethodDeny view base class.
    
    """
    view_classes = http.HttpMethodDeny
    
    def test_included_present_denied(self):
        """
        Check that an included HTTP method name is denied when the handler
        is present.
        
        """
        self.assertViewBehavior(
            {"get": self.unique(), "http_method_names": ("get",)},
            status_code=405)
    
    def test_included_missing_denied(self):
        """
        Check that an included HTTP method name is denied when the handler
        is missing.
        
        """
        self.assertViewBehavior(
            {"http_method_names": ("get",)},
            status_code=405)
    
    def test_excluded_present_denied(self):
        """
        Check that an excluded HTTP method name is denied when the handler
        is present.
        
        """
        self.assertViewBehavior(
            {"get": self.unique(), "http_method_names": ()},
            status_code=405)
    
    def test_exclude_missing_denied(self):
        """
        Check that an excluded HTTP method name is denied when the handler
        is missing.
        
        """
        self.assertViewBehavior(
            {"http_method_names": ()},
            status_code=405)



from __future__ import unicode_literals

import collections

from django.contrib import auth
from django.contrib.auth import models as auth_models
from django.core import exceptions
from django.utils import six

from .. import core


__all__ = (
    "LoginRequired", "ActiveRequired", "StaffRequired", "SuperuserRequired",
    "GroupsRequired", "PermissionsRequired", "ObjectPermissionsRequired",
    "TestRequired",)


class LoginRequired(core.behaviors.Denial):
    """
    A view behavior that tests whether the user is authenticated.
    
    If the login_required attribute is falsy, the login requirement testing
    will be disabled. Its initial value is True.
    
    Set the login_required_* attributes to configure the behavior when a login
    is required in order for the user to proceed. See
    daydreamer.views.core.behaviors.Denial for the attributes' documentation.
    
    """
    login_required = True
    login_required_raise = False
    login_required_exception = None
    login_required_message = None
    login_required_message_level = None
    login_required_message_tags = None
    login_required_redirect_url = None
    login_required_redirect_next_url = None
    login_required_redirect_next_name = auth.REDIRECT_FIELD_NAME
    
    def get_login_required(self):
        """
        A hook to override the login_required value.
        
        The default implementation returns self.login_required.
        
        """
        return self.login_required
    
    def login_required_test(self):
        """
        A hook to override the way that the login requirement test is peformed.
        
        """
        return (
            not self.get_login_required() or 
            self.request.user.is_authenticated())
    
    def login_required_denied(self, request, *args, **kwargs):
        """
        The handler called upon login requirement test failure.
        
        """
        return self.deny("login_required")
    
    def get_deny_handler(self):
        """
        Returns self.login_required_denied when the login requirement
        test fails, falling back to super().
        
        """
        return (
            not self.login_required_test() and
            self.login_required_denied or
            super(LoginRequired, self).get_deny_handler())


class ActiveRequired(core.behaviors.Denial):
    """
    A view behavior that tests whether the user is active.
    
    If the active_required attribute is falsy, the active requirement testing
    will be disabled. Its initial value is True.
    
    Set the active_required_* attributes to configure the behavior when an
    active user is required in order for the user to proceed. See
    daydreamer.views.core.behaviors.Denial for the attributes' documentation.
    
    """
    active_required = True
    active_required_raise = False
    active_required_exception = None
    active_required_message = None
    active_required_message_level = None
    active_required_message_tags = None
    active_required_redirect_url = None
    active_required_redirect_next_url = None
    active_required_redirect_next_name = auth.REDIRECT_FIELD_NAME
    
    def get_active_required(self):
        """
        A hook to override the active_required value.
        
        The default implementation returns self.active_required.
        
        """
        return self.active_required
    
    def active_required_test(self):
        """
        A hook to override the way that the active requirement test
        is performed.
        
        """
        return (
            not self.get_active_required() or
            self.request.user.is_active)
    
    def active_required_denied(self, request, *args, **kwargs):
        """
        The handler called upon active requirement test failure.
        
        """
        return self.deny("active_required")
    
    def get_deny_handler(self):
        """
        Returns self.active_required_denied when the active requirement
        test fails, falling back to super().
        
        """
        return (
            not self.active_required_test() and
            self.active_required_denied or
            super(ActiveRequired, self).get_deny_handler())


class StaffRequired(core.behaviors.Denial):
    """
    A view behavior that tests whether the user is a staff member.
    
    If the staff_required attribute is falsy, the staff requirement testing
    will be disabled. Its initial value is True.
    
    Set the staff_required_* attributes to configure the behavior when a
    staff user is required in order for the user to proceed. See
    daydreamer.views.core.behaviors.Denial for the attributes' documentation.
    
    """
    staff_required = True
    staff_required_raise = False
    staff_required_exception = None
    staff_required_message = None
    staff_required_message_level = None
    staff_required_message_tags = None
    staff_required_redirect_url = None
    staff_required_redirect_next_url = None
    staff_required_redirect_next_name = auth.REDIRECT_FIELD_NAME
    
    def get_staff_required(self):
        """
        A hook to override the staff_required value.
        
        The default implementation returns self.staff_required.
        
        """
        return self.staff_required
    
    def staff_required_test(self):
        """
        A hook to override the way that the active requirement test
        is performed.
        
        """
        return (
            not self.get_staff_required() or
            self.request.user.is_staff)
    
    def staff_required_denied(self, request, *args, **kwargs):
        """
        The handler called upon staff requirement test failure.
        
        """
        return self.deny("staff_required")
    
    def get_deny_handler(self):
        """
        Returns self.staff_required_denied when the staff requirement
        test fails, falling back to super().
        
        """
        return (
            not self.staff_required_test() and
            self.staff_required_denied or
            super(StaffRequired, self).get_deny_handler())


class SuperuserRequired(core.behaviors.Denial):
    """
    A view behavior that tests whether the user is a superuser.
    
    If the superuser_required attribute is falsy, the superuser requirement
    testing will be disabled. Its initial value is True.
    
    Set the superuser_required_* attributes to configure the behavior when
    a superuser is required in order for the user to proceed. See
    daydreamer.views.core.behaviors.Denial for the attributes' documentation.
    
    """
    superuser_required = True
    superuser_required_raise = False
    superuser_required_exception = None
    superuser_required_message = None
    superuser_required_message_level = None
    superuser_required_message_tags = None
    superuser_required_redirect_url = None
    superuser_required_redirect_next_url = None
    superuser_required_redirect_next_name = auth.REDIRECT_FIELD_NAME
    
    def get_superuser_required(self):
        """
        A hook to override the superuser_required value.
        
        The default implementation returns self.superuser_required.
        
        """
        return self.superuser_required
    
    def superuser_required_test(self):
        """
        A hook to override the way that the superuser requirement test
        is performed.
        
        """
        return (
            not self.get_superuser_required() or
            self.request.user.is_superuser)
    
    def superuser_required_denied(self, request, *args, **kwargs):
        """
        The handler called upon superuser requirement test failure.
        
        """
        return self.deny("superuser_required")
    
    def get_deny_handler(self):
        """
        Returns self.superuser_required_denied when the superuser
        requirement test fails, falling back to super().
        
        """
        return (
            not self.superuser_required_test() and
            self.superuser_required_denied or
            super(SuperuserRequired, self).get_deny_handler())


class GroupsRequired(core.behaviors.Denial):
    """
    A view behavior that tests whether the user is in a set of groups.
    
    If the groups_required attribute is falsy, the groups requirement testing
    will be disabled. Its initial value is None. The groups_required attribute
    can be either a single value or an iterable of values. Each value should be
    a group name or a django.contrib.auth.models.Group object. When named
    groups are specified, the corresponding Groups must exist, or a
    django.core.exceptions.ImproperlyConfigured exception will be raised.
    If any values is not a string or a Group object, a ValueError will
    be raised.
    
    Set the groups_required_* attribute to configure the behavior when a set
    of groups is required in order for the user to proceed. See
    daydreamer.views.core.behaviors.Denial for the attributes' documentation.
    
    """
    groups_required = None
    groups_required_raise = False
    groups_required_exception = None
    groups_required_message = None
    groups_required_message_level = None
    groups_required_message_tags = None
    groups_required_redirect_url = None
    groups_required_redirect_next_url = None
    groups_required_redirect_next_name = auth.REDIRECT_FIELD_NAME
    
    def get_groups_required(self):
        """
        A hook to override the groups_required_value.
        
        The default implementation normalizes the groups into a set
        of primary keys. If the groups_required attribute includes a
        value that is not a group name or a
        django.contrib.auth.models.Group object, a ValueError will be raised.
        If any group names do not exist in the database, a
        django.core.exceptions.ImproperlyConfigured exception will be raised.
        
        """
        # Normalize single instances to tuples.
        groups = self.groups_required or set()
        if isinstance(groups, six.string_types + (auth_models.Group,)):
            groups = (groups,)
        elif not isinstance(groups, collections.Iterable):
            raise ValueError(
                "The value {value!r} specified for groups_required is not a "
                "group name, nor a Group nor an iterable of groups.".format(
                    value=groups))
        
        # Resolve the group names and Group objects into existing Groups'
        # primary keys.
        if groups:
            # Filter the groups into buckets by type.
            named_groups = set()
            actual_groups = set()
            for group in groups:
                if isinstance(group, six.string_types):
                    named_groups.add(group)
                elif isinstance(group, auth_models.Group):
                    actual_groups.add(group)
                else:
                    raise ValueError(
                        "A value {value!r} specified in groups_required "
                        "is not a group name or a Group.".format(value=group))
            
            # Resolve the named groups and perform the sanity check.
            resolved_groups = set(
                auth_models.Group.objects
                    .filter(name__in=named_groups)
                    .values_list("pk", flat=True))
            if len(named_groups) != len(resolved_groups):
                raise exceptions.ImproperlyConfigured(
                    "One or more group names specified in groups_required "
                    "does not exist.")
            
            # Gather all the groups' primary keys.
            groups = resolved_groups | set(group.pk for group in actual_groups)
        return groups
    
    def groups_required_test(self):
        """
        A hook to override the way that the groups requirement test
        is performed.
        
        """
        groups = self.get_groups_required()
        return (
            not groups or
            self.request.user.groups.filter(
                pk__in=groups).count() == len(groups))
    
    def groups_required_denied(self, request, *args, **kwargs):
        """
        The handler called upon groups requirement test failure.
        
        """
        return self.deny("groups_required")
    
    def get_deny_handler(self):
        """
        Returns self.groups_required_denied when the groups
        requirement test fails, falling back to super().
        
        """
        return (
            not self.groups_required_test() and
            self.groups_required_denied or
            super(GroupsRequired, self).get_deny_handler())


class PermissionsRequired(core.behaviors.Denial):
    """
    A view behavior that tests whether the user has a set
    of permissions.
    
    If the permissions_required attribute is falsy, the permissions requirement
    testing will be disabled. Its initial value is None. The permissions_required
    attribute can be either a single permission name or an iterable of
    permission names.
    
    Set the permissions_required_* attributes to configure the behavior when
    permissions are required in order for the user to proceed.
    See daydreamer.views.core.behaviors.Denial for the attributes' documentation.
    
    """
    permissions_required = None
    permissions_required_raise = False
    permissions_required_exception = None
    permissions_required_message = None
    permissions_required_message_level = None
    permissions_required_message_tags = None
    permissions_required_redirect_url = None
    permissions_required_redirect_next_url = None
    permissions_required_redirect_next_name = auth.REDIRECT_FIELD_NAME
    
    def get_permissions_required(self):
        """
        A hook to override the permissions_required value.
        
        The default implementation returns the value of
        self.permissions_required, where a single value is normalized as a
        tuple. If any value in permissions_required is not a permission name, a
        ValueError will be raised.
        
        """
        # Normalize single values to a tuple.
        permissions = self.permissions_required or ()
        if isinstance(permissions, six.string_types):
            permissions = (permissions,)
        elif not isinstance(permissions, collections.Iterable):
            raise ValueError(
                "The permssions_required value is neither a permission name "
                "nor an iterable of permission names.")
        
        # Sanity check.
        if (permissions and
            any(not isinstance(permission, six.string_types)
                for permission in permissions)):
            raise ValueError(
                "One or more values in permissions_required is not a "
                "permission name.")
        
        return permissions
    
    def permissions_required_test(self):
        """
        A hook to override the way that the permissions requirement test
        is performed.
        
        """
        permissions = self.get_permissions_required()
        return (
            not permissions or
            self.request.user.has_perms(permissions))
    
    def permissions_required_denied(self, request, *args, **kwargs):
        """
        The handler called upon permissions requirement test failure.
        
        """
        return self.deny("permissions_required")
    
    def get_deny_handler(self):
        """
        Returns self.permissions_required_denied when the permissions
        requirement test fails, falling back to super().
        
        """
        return (
            not self.permissions_required_test() and
            self.permissions_required_denied or
            super(PermissionsRequired, self).get_deny_handler())


class ObjectPermissionsRequired(core.behaviors.Denial):
    """
    A view behavior that tests whether the user has a set of permissions
    for a particular object.
    
    If either of the object_permissions_required attribute or the
    object_permissions_required_object attributes is falsy, the permissions 
    requirement testing will be disabled. Initial values for these attributes
    are None. The object_permissions_required attribute can be either a single
    permission name or an iterable of permission names. The
    object_permissions_required_object attribute will typically be implemented
    as a property that returns some object retrieved from the database.
    
    Set the object_permissions_required_* attributes to configure the behavior
    when permissions are required for an object in order for the user to
    proceed. See daydreamer.views.core.behaviors.Denial for the
    attributes' documentation.
    
    """
    object_permissions_required = None
    object_permissions_required_object = None
    object_permissions_required_raise = False
    object_permissions_required_exception = None
    object_permissions_required_message = None
    object_permissions_required_message_level = None
    object_permissions_required_message_tags = None
    object_permissions_required_redirect_url = None
    object_permissions_required_redirect_next_url = None
    object_permissions_required_redirect_next_name = auth.REDIRECT_FIELD_NAME
    
    def get_object_permissions_required(self):
        """
        A hook to override the object_permissions_required value.
        
        The default implementation returns the value of
        self.object_permissions_required, where a single value is normalized as
        a tuple. If any value in object_permissions_required is not a
        permission name, a ValueError will be raised.
        
        """
        # Normalize single values to a tuple.
        permissions = self.object_permissions_required or ()
        if isinstance(permissions, six.string_types):
            permissions = (permissions,)
        elif not isinstance(permissions, collections.Iterable):
            raise ValueError(
                "The object_permssions_required value is neither a "
                "permission name nor an iterable of permission names.")
        
        # Sanity check.
        if (permissions and
            any(not isinstance(permission, six.string_types)
                for permission in permissions)):
            raise ValueError(
                "One or more values in object_permissions_required is not a "
                "permission name.")
        return permissions
    
    def get_object_permissions_required_object(self):
        """
        A hook to override the object_permissions_required_object value.
        
        The default implementation returns
        self.object_permissions_required_object.
        
        """
        return self.object_permissions_required_object
    
    def object_permissions_required_test(self):
        """
        A hook to override the way that the object permissions requirement test
        is performed.
        
        """
        permissions = self.get_object_permissions_required()
        obj = self.get_object_permissions_required_object()
        return (
            (not all((permissions, obj,))) or
            self.request.user.has_perms(permissions, obj=obj))
    
    def object_permissions_required_denied(self, request, *args, **kwargs):
        """
        The handler called upon object permissions requirement test failure.
        
        """
        return self.deny("object_permissions_required")
    
    def get_deny_handler(self):
        """
        Returns self.object_permissions_required_denied when the object
        permissions test fails, falling back to super().
        
        """
        return (
            not self.object_permissions_required_test() and
            self.object_permissions_required_denied or
            super(ObjectPermissionsRequired, self).get_deny_handler())


class TestRequired(core.behaviors.Denial):
    """
    A view behavior that performs a test against the current request,
    typically a predicate for self.request.user.
    
    If the test_required attribute is not a callable, the test requirement
    will be disabled. Its initial value is None.
    
    Set the test_required_* attributes to configure the behavior when a
    test must be passed in order for the user to proceed. See
    daydreamer.views.core.behaviors.Denial for the attributes' documentation.
    
    """
    test_required = None
    test_required_raise = False
    test_required_exception = None
    test_required_message = None
    test_required_message_level = None
    test_required_message_tags = None
    test_required_redirect_url = None
    test_required_redirect_next_url = None
    test_required_redirect_next_name = auth.REDIRECT_FIELD_NAME
    
    def get_test_required(self):
        """
        A hook to override the test_required value.
        
        The default implementation returns self.test_required.
        
        """
        return self.test_required
    
    def test_required_test(self):
        """
        A hook to override the way that the required test is performed.
        
        """
        test = self.get_test_required()
        return (
            not isinstance(test, collections.Callable) or
            test())
    
    def test_required_denied(self, request, *args, **kwargs):
        """
        The handler called upon test failure.
        
        """
        return self.deny("test_required")
    
    def get_deny_handler(self):
        """
        Returns self.test_required_denied when the test requirement fails,
        falling back to super().
        
        """
        return (
            not self.test_required_test() and
            self.test_required_denied or
            super(TestRequired, self).get_deny_handler())



from __future__ import unicode_literals

import collections

from django.utils import six
from django.utils.decorators import classonlymethod
from django.views.decorators import debug

from .. import generic


__all__ = ("SensitiveVariables", "SensitivePostParameters",)


class SensitiveVariables(generic.View):
    """
    A view behavior that marks specified variable names as sensitive so that
    their values are not displayed by Django's error logging mechanisms in
    production (when settings.DEBUG == False).
    
    Set sensitive_variables to a string or iterable of strings to mark specific
    variable names as sensitive. Set it to a truthy value to mark all variable
    names as sensitive. Set it to a falsy value to disable the behavior. Its
    default value is True to protect all variables.
    
    See the django.views.decorators.debug.sensitive_variables() decorator
    for details.
    
    """
    sensitive_variables = True
    
    @classonlymethod
    def as_view(cls, **kwargs):
        """
        Optionally wrap the base view in
        django.views.decorators.debug.sensitive_variables().
        
        """
        # Normalize the setting.
        variables = cls.sensitive_variables
        if isinstance(variables, six.string_types):
            variables = (variables,)
        elif isinstance(variables, collections.Iterable):
            variables = tuple(variables)
        elif variables:
            variables = ()
        else:
            variables = None
        
        view = super(SensitiveVariables, cls).as_view(**kwargs)
        return (
            debug.sensitive_variables(*variables)(view)
                if variables is not None
                else view)


class SensitivePostParameters(generic.View):
    """
    A view behavior that marks specified POST parameter names as sensitive so
    that their values are not displayed by Django's error logging mechanisms
    in production (when settings.DEBUG == False).
    
    Set sensitive_postt_parameters to a string or iterable of strings to mark
    specific POST parameter names as sensitive. Set it to a truthy value to
    mark all POST parameter names as sensitive. Set it to a falsy value to
    disable the behavior. Its default value is True to protect all
    POST parameters.
    
    See the django.views.decorators.debug.sensitive_post_parameters() decorator
    for details.
    
    """
    sensitive_post_parameters = True
    
    @classonlymethod
    def as_view(cls, **kwargs):
        """
        Optionally wrap the base view in
        django.views.decorators.debug.sensitive_post_parameters().
        
        """
        # Normalize the setting.
        parameters = cls.sensitive_post_parameters
        if isinstance(parameters, six.string_types):
            parameters = (parameters,)
        elif isinstance(parameters, collections.Iterable):
            parameters = tuple(parameters)
        elif parameters:
            parameters = ()
        else:
            parameters = None
        
        view = super(SensitivePostParameters, cls).as_view(**kwargs)
        return (
            debug.sensitive_post_parameters(*parameters)(view)
                if parameters is not None
                else view)



from __future__ import unicode_literals

from django.utils.decorators import classonlymethod
from django.views.decorators import clickjacking

from .. import generic


__all__ = (
    "XFrameOptionsDeny", "XFrameOptionsSameOrigin", "XFrameOptionsExempt",)


class XFrameOptionsDeny(generic.View):
    """
    A view behavior that adds a DENY X-Frame-Options header to the response
    at the top level of the dispatch.
    
    Set the xframe_options_deny attribute to a falsy value to disable
    the behavior.
    
    See the django.views.decorators.clickjacking.xframe_options_deny()
    decorator for details.
    
    """
    xframe_options_deny = True
    
    @classonlymethod
    def as_view(cls, **kwargs):
        """
        Optionally decorates the base view with the
        django.views.decorators.clickjacking.xframe_options_deny() decorator.
        
        """
        view = super(XFrameOptionsDeny, cls).as_view(**kwargs)
        return (
            clickjacking.xframe_options_deny(view)
                if cls.xframe_options_deny
                else view)


class XFrameOptionsSameOrigin(generic.View):
    """
    A view behavior that adds a SAMEORIGIN X-Frame-Options header to the
    response at the top level of the dispatch.
    
    Set the xframe_options_same_origin attribute to a falsy value to disable
    the behavior.
    
    See the django.views.decorators.clickjacking.xframe_options_sameorigin()
    decorator for details.
    
    """
    xframe_options_same_origin = True
    
    @classonlymethod
    def as_view(cls, **kwargs):
        """
        Optionally decorates the base view with the
        django.views.decorators.clickjacking.xframe_options_sameorigin()
        decorator.
        
        """
        view = super(XFrameOptionsSameOrigin, cls).as_view(**kwargs)
        return (
            clickjacking.xframe_options_sameorigin(view)
                if cls.xframe_options_same_origin
                else view)


class XFrameOptionsExempt(generic.View):
    """
    A view behavior that marks the response as exempt from protection so that
    the middleware does not add an X-Frame-Options header.
    
    Set the xframe_options_exempt attribute to a falsy value to disable
    the behavior.
    
    See the django.views.decorators.clickjacking.xframe_options_exempt()
    decorator for details.
    
    """
    xframe_options_exempt = True
    
    @classonlymethod
    def as_view(cls, **kwargs):
        """
        Optionally decorates the base view with the
        django.views.decorators.clickjacking.xframe_options_exempt() decorator.
        
        """
        view = super(XFrameOptionsExempt, cls).as_view(**kwargs)
        return (
            clickjacking.xframe_options_exempt(view)
                if cls.xframe_options_exempt
                else view)



from __future__ import unicode_literals

import functools

from . import base


class TestCase(base.TestCase):
    """
    A test case for making assertions about messsages from
    django.contrib.messages in the context of a response.
    
    """
    def assertMessage(self, response, content, level=None, tags=None,
            limit=None):
        """
        Asserts that the response has a particular message in its context.
        
        If limit is provided, checks that there are at most the specified
        number of messages.
        
        If level or tags are specified, checks for existence of message having
        matching content, level and/or tags. Otherwise, it simply checks for a
        message with the content.
        
        """
        # There must be a context in the response.
        self.assertTrue(
            hasattr(response, "context") and response.context,
            "The response must have a non-empty context attribute.")
        
        # Get the messages from the context. This may be a fragile
        # django.test.utils.ContextList data structure, so we can't
        # call .get() on it.
        messages = list(
            response.context["messages"]
                if "messages" in response.context
                else [])
        
        # Check for at least one message.
        self.assertTrue(
            bool(messages),
            "The response's context must contain at least one message.")
        
        # Check for a message limit.
        if limit:
            self.assertGreaterEqual(
                limit, len(messages),
                "The response's context must have at most {limit:d} "
                "messages, but it has {actual:d} messages.".format(
                    limit=limit, actual=len(messages)))
        
        # Check for a matching message.
        self.assertTrue(
            any((
                message.message == content and
                (not level or message.level == level) and
                (not tags or (
                    set(tag.strip() 
                        for tag in (message.tags or "").split(" ")
                        if tag) == set(tag.strip()
                            for tag in (tags or "").split(" ")
                            if tag))))
                for message in messages),
            "A message matching the content, level and tags was not found in "
            "the response's context.")
    
    def assertNoMessages(self, response):
        """
        Asserts that the response does not have any messages in its context.
        
        """
        if getattr(response, "context", None):
            # The context may be a fragile django.test.utils.ContextList data
            # structure, so we can't call .get() on it.
            self.assertEqual(
                len(list(
                    response.context["messages"]
                        if "messages" in response.context
                        else [])), 0,
                "The response's context must not contain any messages.")
    
    def create_message_assertions(self, content=None, level=None, tags=None,
            limit=None):
        """
        Returns a tuple of message assertion callbacks for the given arguments.
        
        """
        return (
            (functools.partial(self.assertMessage,
                content=content, level=level, tags=tags, limit=limit),)
                if content is not None
                else (self.assertNoMessages,))
    
    def create_response_assertions(self, message=None, message_level=None,
            message_tags=None, message_limit=None, **kwargs):
        """
        Adds message assertions to the response assertions.
        
        """
        return (
            self.create_message_assertions(
                content=message,
                level=message_level,
                tags=message_tags,
                limit=message_limit) +
            super(TestCase, self).create_response_assertions(**kwargs))



from __future__ import unicode_literals

import collections
import functools

from . import base


class TestCase(base.TestCase):
    """
    A test case for making assertions about attributes on a view.
    
    """
    def assertViewIncludes(self, view, names):
        """
        Asserts that the view contains the given attribute name(s).
        
        """
        self.assertAttributesIn(names, view)
    
    def assertViewExcludes(self, view, names):
        """
        Asserts that the view does not contain the given attribute name(s).
        
        """
        self.assertAttributesNotIn(names, view)
    
    def assertViewExact(self, view, attrs):
        """
        Asserts that the view contains the attribute names and values
        specified by attrs.
        
        """
        self.assertAttributesExact(view, attrs)
    
    def create_view_attribute_assertions(self, include=None, exclude=None,
            exact=None):
        """
        Returns a tuple of view assertion callbacks for the given arguments.
        
        """
        include = include or ()
        exclude = exclude or ()
        exact = (
            exact
                if isinstance(exact, collections.Mapping)
                else dict(exact or {}))
        return (
            ((functools.partial(self.assertViewIncludes, names=include),)
                if include
                else ()) +
            ((functools.partial(self.assertViewExcludes, names=exclude),)
                if exclude
                else ()) +
            ((functools.partial(self.assertViewExact, attrs=exact),)
                if exact
                else ()))
    
    def create_method_assertions(self, view_includes=None, view_excludes=None,
            view_exact=None, **kwargs):
        """
        Adds view attribute assertions to the method assertions.
        
        """
        return (
            self.create_view_attribute_assertions(
                include=view_includes,
                exclude=view_excludes,
                exact=view_exact) +
            super(TestCase, self).create_method_assertions(**kwargs))



from __future__ import unicode_literals

import unittest

from django.core import exceptions

from daydreamer.views import generic
from daydreamer.views.behaviors import auth as auth

from . import base


class LoginRequiredTestCase(base.TestCase):
    """
    Tests for the LoginRequired view behavior.
    
    """
    view_classes = (auth.LoginRequired, generic.View,)
    prefix = "login_required"
    
    def setup_unauth_fail(self):
        return {}
    
    def setup_auth_fail(self):
        # All authenticated users should pass, so defer to the unauthenticated
        # failure setup.
        return self.setup_unauth_fail()
    
    def setup_unauth_pass(self):
        return {"": False}
    
    def setup_auth_pass(self):
        self.create_authenticated_user()
        return {}


class ActiveRequiredTestCase(base.TestCase):
    """
    Tests for the ActiveRequired view behavior.
    
    """
    view_classes = (auth.ActiveRequired, generic.View,)
    prefix = "active_required"
    
    def setup_unauth_fail(self):
        return {}
    
    def setup_auth_fail(self):
        self.create_authenticated_user(is_active=False)
        return {}
    
    def setup_unauth_pass(self):
        return {"": False}
    
    def setup_auth_pass(self):
        self.create_authenticated_user()
        return {}


class StaffRequiredTestCase(base.TestCase):
    """
    Tests for the StaffRequired view behavior.
    
    """
    view_classes = (auth.StaffRequired, generic.View,)
    prefix = "staff_required"
    
    def setup_unauth_fail(self):
        return {}
    
    def setup_auth_fail(self):
        self.create_authenticated_user()
        return {}
    
    def setup_unauth_pass(self):
        return {"": False}
    
    def setup_auth_pass(self):
        self.create_authenticated_user(is_staff=True)
        return {}


class SuperuserRequiredTestCase(base.TestCase):
    """
    Tests for the SuperuserRequired view behavior.
    
    """
    view_classes = (auth.SuperuserRequired, generic.View,)
    prefix = "superuser_required"
    
    def setup_unauth_fail(self):
        return {}
    
    def setup_auth_fail(self):
        self.create_authenticated_user()
        return {}
    
    def setup_unauth_pass(self):
        return {"": False}
    
    def setup_auth_pass(self):
        self.create_authenticated_user(is_superuser=True)
        return {}


class GroupsRequiredTestCase(base.TestCase):
    """
    Tests for the GroupsRequired view behavior.
    
    """
    view_classes = (auth.GroupsRequired, generic.View,)
    prefix = "groups_required"
    
    def setup_unauth_fail(self):
        group = self.create_group()
        return {"": group.name}
    
    def setup_auth_fail(self):
        self.create_authenticated_user()
        group = self.create_group()
        return {"": group.name}
    
    def setup_unauth_pass(self):
        return {"": None}
    
    def setup_auth_pass(self):
        user = self.create_authenticated_user()
        group = self.create_group()
        user.groups.add(group)
        return {"": group.name}
    
    # Setups for multiple groups required.
    def setup_unauth_fail_multiple(self):
        group1, group2 = groups = (
            self.create_group(), self.create_group(),)
        return {"": (group1.name, group2)}
    
    def setup_auth_fail_multiple(self):
        user = self.create_authenticated_user()
        group1, group2 = groups = (
            self.create_group(), self.create_group(),)
        user.groups.add(group1)
        return {"": (group1.name, group2)}
    
    def setup_auth_pass_multiple(self):
        user = self.create_authenticated_user()
        group1, group2 = groups = (
            self.create_group(), self.create_group(),)
        map(user.groups.add, groups)
        return {"": (group1.name, group2)}
    
    # Tests for multiple groups required.
    def test_unauth_fail_multiple(self):
        return self.test_unauth_fail(self.setup_unauth_fail_multiple)
    
    def test_auth_fail_multiple(self):
        return self.test_auth_fail(self.setup_auth_fail_multiple)
    
    def test_auth_pass_multiple(self):
        return self.test_auth_pass(self.setup_auth_pass_multiple)
    
    # Tests for invalid group values.
    def test_invalid_type(self):
        with self.assertRaises(ValueError):
            self.client.get(self.view(**{"": 1}))
    
    def test_invalid_type_multiple(self):
        with self.assertRaises(ValueError):
            self.client.get(self.view(**{"": (1, 1)}))
    
    def test_invalid_group(self):
        with self.assertRaises(exceptions.ImproperlyConfigured):
            self.client.get(self.view(**{"": self.unique()}))
    
    def test_invalid_group_multiple(self):
        with self.assertRaises(exceptions.ImproperlyConfigured):
            self.client.get(self.view(**{"": (self.unique(), self.unique(),)}))


class PermissionsRequiredTestCase(base.TestCase):
    """
    Tests for the PermissionsRequired view behavior.
    
    """
    view_classes = (auth.PermissionsRequired, generic.View,)
    prefix = "permissions_required"
    
    def setup_unauth_fail(self):
        permission = self.create_permission()
        return {"": ".".join(("auth", permission.codename,))}
    
    def setup_auth_fail(self):
        permission = self.create_permission()
        self.create_authenticated_user()
        return {"": ".".join(("auth", permission.codename,))}
    
    def setup_unauth_pass(self):
        return {}
    
    def setup_auth_pass(self):
        permission = self.create_permission()
        user = self.create_authenticated_user()
        user.user_permissions.add(permission)
        return {"": ".".join(("auth", permission.codename,))}
    
    # Setups for multiple permissions.
    def setup_unauth_fail_multiple(self):
        permissions = self.create_permission(), self.create_permission()
        return {"": [
            ".".join(("auth", permission.codename,))
            for permission in permissions]}
    
    def setup_auth_fail_multiple(self):
        permissions = permission1, permission2 = (
            self.create_permission(), self.create_permission())
        user = self.create_authenticated_user()
        user.user_permissions.add(permission1)
        return {"": [
            ".".join(("auth", permission.codename,))
            for permission in permissions]}
    
    def setup_auth_pass_multiple(self):
        permissions = self.create_permission(), self.create_permission()
        user = self.create_authenticated_user()
        map(user.user_permissions.add, permissions)
        return {"": [
            ".".join(("auth", permission.codename,))
            for permission in permissions]}
    
    # Tests for multiple groups required.
    def test_unauth_fail_multiple(self):
        return self.test_unauth_fail(self.setup_unauth_fail_multiple)
    
    def test_auth_fail_multiple(self):
        return self.test_auth_fail(self.setup_auth_fail_multiple)
    
    def test_auth_pass_multiple(self):
        return self.test_auth_pass(self.setup_auth_pass_multiple)
    
    # Tests for invalid permission values.
    def test_invalid(self):
        with self.assertRaises(ValueError):
            self.client.get(self.view(**{"": 1}))
    
    def test_invalid_multiple(self):
        with self.assertRaises(ValueError):
            self.client.get(self.view(**{"": (1, 1)}))


class ObjectPermissionsRequiredTestCase(base.TestCase):
    """
    Tests for the ObjectPermissionsRequired view behavior.
    
    """
    view_classes = (auth.ObjectPermissionsRequired, generic.View,)
    prefix = "object_permissions_required"
    
    def setup_unauth_fail(self):
        permission = self.create_permission()
        return {
            "": ".".join(("auth", permission.codename,)),
            "object": object()}
    
    def setup_auth_fail(self):
        permission = self.create_permission()
        self.create_authenticated_user()
        return {
            "": ".".join(("auth", permission.codename,)),
            "object": object()}
    
    def setup_unauth_pass(self):
        return {}
    
    def setup_auth_pass(self):
        permission = self.create_permission()
        obj = object()
        user = self.create_authenticated_user()
        # With an object permissions system, the permissions should be for
        # the object.
        user.user_permissions.add(permission)
        return {
            "": ".".join(("auth", permission.codename,)),
            "object": obj}
    
    # Setups for multiple permissions.
    def setup_unauth_fail_multiple(self):
        permissions = self.create_permission(), self.create_permission()
        return {
            "": [
                ".".join(("auth", permission.codename,))
                for permission in permissions],
            "object": object()}
    
    def setup_auth_fail_multiple(self):
        permissions = permission1, permission2 = (
            self.create_permission(), self.create_permission())
        user = self.create_authenticated_user()
        # With an object permissions system, the permissions should be for
        # the object.
        user.user_permissions.add(permission1)
        return {
            "": [
                ".".join(("auth", permission.codename,))
                for permission in permissions],
            "object": object()}
    
    def setup_auth_pass_multiple(self):
        permissions = self.create_permission(), self.create_permission()
        obj = object()
        user = self.create_authenticated_user()
        # With an object permissions system, the permissions should be for
        # the object.
        map(user.user_permissions.add, permissions)
        return {
            "": [
                ".".join(("auth", permission.codename,))
                for permission in permissions],
            "object": obj}
    
    # Tests that cannot pass without an object permissions system.
    @unittest.expectedFailure
    def test_auth_pass(self, setup_auth_pass=None):
        return (
            super(ObjectPermissionsRequiredTestCase, self)
                .test_auth_pass(setup_auth_pass))
    
    @unittest.expectedFailure
    def test_auth_pass_raise(self, setup_auth_pass=None):
        return (
            super(ObjectPermissionsRequiredTestCase, self)
                .test_auth_pass_raise(setup_auth_pass))
    
    @unittest.expectedFailure
    def test_auth_pass_message(self, setup_auth_pass=None):
        return (
            super(ObjectPermissionsRequiredTestCase, self)
                .test_auth_pass_message(setup_auth_pass))
    
    @unittest.expectedFailure
    def test_auth_pass_precedence(self, setup_auth_pass=None):
        return (
            super(ObjectPermissionsRequiredTestCase, self)
                .test_auth_pass(setup_auth_pass))
    
    @unittest.expectedFailure
    def test_auth_pass_multiple(self):
        return self.test_auth_pass(self.setup_auth_pass_multiple)
    
    # Tests for multiple groups required.
    def test_unauth_fail_multiple(self):
        return self.test_unauth_fail(self.setup_unauth_fail_multiple)
    
    def test_auth_fail_multiple(self):
        return self.test_auth_fail(self.setup_auth_fail_multiple)
    
    # Tests for invalid permission values.
    def test_invalid(self):
        with self.assertRaises(ValueError):
            self.client.get(self.view(**{"": 1, "object": object()}))
    
    def test_invalid_multiple(self):
        with self.assertRaises(ValueError):
            self.client.get(self.view(**{"": (1, 1), "object": object()}))


class TestRequiredTestCase(base.TestCase):
    """
    Tests for the TestRequired view behavior.
    
    """
    view_classes = (auth.TestRequired, generic.View,)
    prefix = "test_required"
    
    def create_name_test(self, name):
        def test_required(self):
            return getattr(self.request.user, "first_name", None) == name
        return test_required
    
    def setup_unauth_fail(self):
        return {"": self.create_name_test(self.unique_username())}
    
    def setup_auth_fail(self):
        self.create_authenticated_user()
        return {"": self.create_name_test(self.unique_username())}
    
    def setup_unauth_pass(self):
        return {}
    
    def setup_auth_pass(self):
        name = self.unique_username()
        self.create_authenticated_user(first_name=name)
        return {"": self.create_name_test(name)}



from __future__ import unicode_literals

from django.test import client

from daydreamer.core import handlers


class ClientHandler(client.ClientHandler, handlers.base.Handler):
    """
    A customized test client handler that keeps track of the most recently
    generated request.
    
    """
    def __init__(self, *args, **kwargs):
        super(ClientHandler, self).__init__(*args, **kwargs)
        self.last_request = None
    
    @property
    def last_request(self):
        return self._last_request
    
    @last_request.setter
    def last_request(self, request):
        self._last_request = request
    
    def get_response(self, request):
        self.last_request = request
        return super(ClientHandler, self).get_response(request)



from __future__ import unicode_literals

import collections

from django.utils import six
from django.views.decorators import vary

from .. import core


__all__ = ("VaryOnHeaders", "VaryOnCookie",)


class VaryOnHeaders(core.http.HttpMethodAllow):
    """
    A view behavior that adds values to the response's Vary header during the
    allow phase of the dispatch.
    
    Specify the case-insensitive Vary header value or values to add to response
    in the vary_on_headers attribute. If none are specified, the behavior will
    be disabled.
    
    See the django.views.decorators.vary.vary_on_headers() view decorator
    for details.
    
    """
    vary_on_headers = None
    
    def get_vary_on_headers(self):
        """
        A hook to control the Vary headers.
        
        """
        return self.vary_on_headers
    
    def get_allow_handler(self):
        """
        If Vary headers are specified, wrap the base allow handler in the
        vary_on_headers() decorator.
        
        """
        # Noramlize the Vary header values.
        vary_on_headers = self.get_vary_on_headers()
        if isinstance(vary_on_headers, six.string_types):
            vary_on_headers = (vary_on_headers,)
        
        allow = super(VaryOnHeaders, self).get_allow_handler()
        if vary_on_headers:
            allow = vary.vary_on_headers(*vary_on_headers)(allow)
        return allow


class VaryOnCookie(core.http.HttpMethodAllow):
    """
    A view behavior that adds "Cookie" to the response's Vary header during the
    allow phase of the dispatch.
    
    Set the vary_on_cookie to a falsy value to disable the behavior.
    
    See the django.views.decorators.vary_on_cookie() view decorator
    for details.
    
    """
    vary_on_cookie = True
    
    def get_vary_on_cookie(self):
        """
        A hook to control whether to add "Cookie" to the Vary header.
        
        """
        return self.vary_on_cookie
    
    def get_allow_handler(self):
        """
        If specified, wrap the base allow handler in the
        vary_on_cookie() decorator.
        
        """
        vary_on_cookie = self.get_vary_on_cookie()
        allow = super(VaryOnCookie, self).get_allow_handler()
        if vary_on_cookie:
            allow = vary.vary_on_cookie(allow)
        return allow



from __future__ import unicode_literals

from daydreamer.views.behaviors import gzip

from . import base


class GZipPageTestCase(base.TestCase):
    """
    Tests for the GZipPage view behavior.
    
    """
    view_classes = gzip.GZipPage
    
    def test_gzip_page(self):
        """
        Check that a response's content is gzipped when the conditions for
        gzipping are correct.
        
        """
        content = self.unique_gzip()
        self.assertViewBehavior(
            {"get": content},
            headers={"HTTP_ACCEPT_ENCODING": "gzip"},
            status_code=200,
            content=self.compress(content),
            headers_exact={"Content-Encoding": "gzip"})
    
    def test_gzip_page_disabled(self):
        """
        Check that the response's content is not gzipped when the view behavior
        is disabled, even when the conditions for gzipping are correct.
        
        """
        content = self.unique_gzip()
        self.assertViewBehavior(
            {"gzip_page": False, "get": content},
            headers={"HTTP_ACCEPT_ENCODING": "gzip"},
            status_code=200,
            content=content,
            headers_exclude="Content-Encoding")
    
    def test_gzip_page_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence and
        that the response's content is not gzipped, because the content is
        too short.
        
        """
        self.assertViewBehavior(
            headers={"HTTP_ACCEPT_ENCODING": "gzip"},
            status_code=405,
            headers_exclude="Content-Encoding")
    
    def test_gzip_page_disabled_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence and
        that the response's context is not gzipped when the view behavior is
        disabled, even when the conditions for gzipping are correct.
        
        """
        self.assertViewBehavior(
            {"gzip_page": False},
            headers={"HTTP_ACCEPT_ENCODING": "gzip"},
            status_code=405,
            headers_exclude="Content-Encoding")
    


from . import auth, cache, clickjacking, csrf, http, vary
from .auth import (LoginRequired, ActiveRequired, StaffRequired,
    SuperuserRequired, GroupsRequired, PermissionsRequired,
    ObjectPermissionsRequired, TestRequired,)
from .cache import CachePage, CacheControl, NeverCache
from .clickjacking import (XFrameOptionsDeny, XFrameOptionsSameOrigin,
    XFrameOptionsExempt)
from .csrf import CsrfProtect, RequiresCsrfToken, EnsureCsrfCookie, CsrfExempt
from .debug import SensitiveVariables, SensitivePostParameters
from .http import RequireGET, RequirePOST, RequireSafe, Condition
from .gzip import GZipPage
from .vary import VaryOnHeaders, VaryOnCookie


__all__ = (
    "LoginRequired", "ActiveRequired", "StaffRequired", "SuperuserRequired",
    "GroupsRequired", "PermissionsRequired", "ObjectPermissionsRequired",
    "TestRequired",
    "CachePage", "CacheControl", "NeverCache",
    "XFrameOptionsDeny", "XFrameOptionsSameOrigin", "XFrameOptionsExempt",
    "CsrfProtect", "RequiresCsrfToken", "EnsureCsrfCookie", "CsrfExempt",
    "SensitiveVariables", "SensitivePostParameters",
    "RequireGET", "RequirePOST", "RequireSafe", "Condition",
    "GZipPage",
    "VaryOnHeaders", "VaryOnCookie",)



from __future__ import unicode_literals

from django.conf import settings

from daydreamer.tests.views import generic


class TestCase(generic.TestCase):
    """
    Common utilities for testing clickjacking view behaviors.
    
    Clickjacking middleware is disabled by default with the
    clickjacking_middleware_enabled attribute.
    
    """
    clickjacking_middleware_enabled = False
    
    def setUp(self):
        if not self.clickjacking_middleware_enabled:
            self._clickjacking_middleware = settings.MIDDLEWARE_CLASSES
            settings.MIDDLEWARE_CLASSES = tuple(
                middleware for middleware in settings.MIDDLEWARE_CLASSES
                if middleware !=
                    "django.middleware.clickjacking.XFrameOptionsMiddleware")
        super(TestCase, self).setUp()
    
    def tearDown(self):
        super(TestCase, self).tearDown()
        if not self.clickjacking_middleware_enabled:
            settings.MIDDLEWARE_CLASSES = self._clickjacking_middleware
            del self._clickjacking_middleware



from django.core.handlers import wsgi

from . import base


class WSGIHandler(wsgi.WSGIHandler, base.Handler):
    """
    A WSGI handler, enhanced with object-oriented hooks for response generation
    from daydreamer.core.handlers.base.Handler.
    
    """
    pass



from __future__ import unicode_literals

from django.conf import settings

from daydreamer.tests.views import generic

from . import client


class TestCase(generic.TestCase):
    """
    Common utilities for testing CSRF view behaviors.
    
    CSRF middleware is disabled by default with the
    csrf_middleware_enabled attribute. CSRF processing during tests is
    enabled by default, and a specialized client for managing CSRF
    cookies is provided.
    
    """
    client_class = client.Client
    csrf_middleware_enabled = False
    enforce_csrf_checks = True
    
    def setUp(self):
        if not self.csrf_middleware_enabled:
            self._csrf_middleware = settings.MIDDLEWARE_CLASSES
            settings.MIDDLEWARE_CLASSES = tuple(
                middleware for middleware in settings.MIDDLEWARE_CLASSES
                if middleware != "django.middleware.csrf.CsrfViewMiddleware")
        super(TestCase, self).setUp()
    
    def tearDown(self):
        super(TestCase, self).tearDown()
        if not self.csrf_middleware_enabled:
            settings.MIDDLEWARE_CLASSES = self._csrf_middleware
            del self._csrf_middleware



from __future__ import unicode_literals

from .. import handler


class ClientHandler(handler.ClientHandler):
    """
    A customized test client handler that skips over the URL resolution system
    and runs a view specified in the request's META data under the keys
    "django.view", "django.view_args" and "django.view_kwargs".
    
    """
    def get_resolver_match(self, request, resolver):
        """
        Resolves the view from META data set on the request. Returns a
        triple containing the view, its args and its kwargs.
        
        """
        return (
            request.META.get("django.view"),
            request.META.get("django.view_args", ()),
            request.META.get("django.view_kwargs", {}),)



from __future__ import unicode_literals

from django.utils.decorators import classonlymethod
from django.views.decorators import csrf

from .. import generic


__all__ = (
    "CsrfProtect", "RequiresCsrfToken", "EnsureCsrfCookie", "CsrfExempt",)


class CsrfProtect(generic.View):
    """
    A view behavior that adds CSRF protection to a view at the top level of
    the dispatch.
    
    Disable the view decorator's functionality by setting the csrf_protect
    attribute to a falsy value. Note that this is not the same as making the
    view exempt from CSRF checks (see CsrfExempt).
    
    """
    csrf_protect = True
    
    @classonlymethod
    def as_view(cls, **kwargs):
        """
        Optionally decorates the base view function with
        django.views.decorators.csrf.csrf_protect().
        
        """
        view = super(CsrfProtect, cls).as_view(**kwargs)
        return csrf.csrf_protect(view) if cls.csrf_protect else view


class RequiresCsrfToken(generic.View):
    """
    A view behavior that ensures "csrf_token" will be present in the request
    context, but does not enforce CSRF protection at the top level of
    the dispatch.
    
    Disable the view decorator's functionality by setting the
    requires_csrf_token attribute to a falsy value.
    
    """
    requires_csrf_token = True
    
    @classonlymethod
    def as_view(cls, **kwargs):
        """
        Optionally decorates the base view function with
        django.views.decorators.csrf.requires_csrf_token().
        
        """
        view = super(RequiresCsrfToken, cls).as_view(**kwargs)
        return (
            csrf.requires_csrf_token(view)
                if cls.requires_csrf_token
                else view)


class EnsureCsrfCookie(generic.View):
    """
    A view behavior that ensures a CSRF cookie will be present in the response,
    but does not enforce CSRF protection at the top level of the dispatch.
    
    Disable the view decorator's functionality by setting the
    ensure_csrf_cookie attribute to a falsy value.
    
    """
    ensure_csrf_cookie = True
    
    @classonlymethod
    def as_view(cls, **kwargs):
        """
        Optionally decorates the base view function with
        django.views.decorators.csrf.ensure_csrf_cookie().
        
        """
        view = super(EnsureCsrfCookie, cls).as_view(**kwargs)
        return (
            csrf.ensure_csrf_cookie(view)
                if cls.ensure_csrf_cookie
                else view)


class CsrfExempt(generic.View):
    """
    A view behavior that disables CSRF protection at the top level of
    the dispatch.
    
    Disable the view decorator's functionality by setting the
    csrf_exampt attribute to a falsy value.
    
    """
    csrf_exempt = True
    
    @classonlymethod
    def as_view(cls, **kwargs):
        """
        Optionally decorates the base view function with
        django.views.decorators.csrf.csrf_exempt().
        
        """
        view = super(CsrfExempt, cls).as_view(**kwargs)
        return csrf.csrf_exempt(view) if cls.csrf_exempt else view



from __future__ import unicode_literals

import collections
import copy as copying

from django.utils import six


__all__ = ("updated",)


def updated(destination, source, copy=False):
    """
    Updates the destination dictionary or dictionary-like object with the
    source dictionary. Optionally makes a shallow copy of the destination
    before the update. Returns the updated dictionary.
    
    """
    if copy:
        if isinstance(
            getattr(destination, "copy", None), collections.Callable):
            destination = destination.copy()
        else:
            destination = copying.copy(destination)
    
    if isinstance(destination, collections.MutableMapping):
        destination.update(source)
    else:
        for key, value in six.iteritems(
                source
                    if isinstance(source, collections.Mapping)
                    else dict(source)):
            destination[key] = value
    return destination


def any(values, falsy=False):
    """
    Returns the first truthy value encountered in the values iterable,
    falling back to the given falsy value.
    
    """
    for value in values:
        if value:
            return value
    return falsy


def all(values, truthy=True):
    """
    Returns the first falsy value encountered in the values interable,
    falling back to the given truthy value.
    
    """
    for value in values:
        if not value:
            return value
    return truthy




from __future__ import unicode_literals

from django import http
from django.conf import settings
from django.contrib import messages
from django.core import exceptions

from daydreamer.core import urlresolvers

from . import base


__all__ = ("Denial",)


class Denial(base.Deny):
    """
    An abstract base class providing a denial behavior framework.
    
    Implements the deny() method, which when called:
    
        * If exceptions are enabled, raises the configured exception.
        * If a denial message has been specified, enqueues a message with the
          django.contrib.messages framework.
        * Finally, redirects to a denial URL with an optional query string
          specifying the next URL, i.e. to redirect back when desired.
    
    This encapsulates common view patterns and makes them configurable
    through a declarative API, facilitating the implementation of many
    view behaviors and helping you avoid repetition of boilerplate code. The
    view behaviors in daydreamer.views.behaviors make heavy use of these
    features and can simplify your view implementations.
    
    The deny() method should be called with a prefix name that it
    will use to retrieve attributes that customize its behavior as follows:
        
        When the <prefix>_raise attribute is set to a truthy value, raises
        <prefix>_exception.
        
        Set the <prefix>_exception attribute to an exception class or
        instance to raise. Defaults to django.core.exceptions.PermissionDenied
        when falsy.
        
        Set the <prefix>_message attribute to a message to enqueue via the
        django.contrib.messages framework. If falsy, no message will
        be enqueued.
        
        Set the <prefix>_message_level attribute to the level of the failure
        message, a value from django.contrib.messages.constants. Defaults to
        WARNING when falsy.
        
        Set the <prefix>_message_tags attribute to a space-separated string
        specifying additional tags to add to the message, typically used for
        CSS styling purposes. Defaults to the empty string when falsy.
        
        Set the <prefix>_redirect_url attribute to the failure redirect URL.
        Defaults to settings.LOGIN_URL when falsy. The URL must be a string or
        a lazy string, such as the result of
        django.core.urlresovlers.reverse_lazy(). However, settings.LOGIN_URL
        may be a named URL pattern, as documented for Django's settings.
        
        Set the <prefix>_redirect_next_url attribute to the URL of the value
        for the query string parameter to append to the redirect URL. Defaults
        to the request's fully-qualified URL when falsy.
        
        Set the <prefix>_redirect_next_name attribute to the name of the
        query string parameter to append to the redirect URL for the value
        of the next URL. If None, no query string parameter will be added to
        the redirect URL. A typical value would be
        django.contrib.auth.REDIRECT_FIELD_NAME.
        
    Additional object-oriented hooks are provided by the implementation. See
    the source code for details.
    
    """
    # Hooks for resolving attribute values used by deny().
    def get_denial_attr(self, prefix, attr):
        """
        A hook to customize the way that attributes for the deny()
        method are retrieved from a prefix and attribute name.
        
        The default implementation joins the prefix and attribute name with
        "_" and looks up the attribute with getattr. This will raise an
        AttributeError if the generated attribute name does not exist.
        
        """
        return getattr(self, "_".join((prefix, attr)))
    
    def get_denial_raise(self, prefix):
        """
        A hook to customize resolution of the exception raising setting
        used by deny().
        
        The default implementation returns self.<prefix>_raise.
        
        """
        return self.get_denial_attr(prefix, "raise")
    
    def get_denial_exception(self, prefix):
        """
        A hook to customize resolution of the exception value to raise
        used by deny().
        
        The default implementation returns self.<prefix>_exception,
        defaulting to django.core.exceptions.PermissionDenied when falsy.
        
        """
        return (
            self.get_denial_attr(prefix, "exception") or
            exceptions.PermissionDenied)
    
    def get_denial_message(self, prefix):
        """
        A hook to customize resolution of the message value to enqueue,
        used by deny().
        
        The default implementation returns self.<prefix>_message.
        
        """
        return self.get_denial_attr(prefix, "message")
    
    def get_denial_message_level(self, prefix):
        """
        A hook to customize resolution of the message level used
        by deny().
        
        The default implementation returns self.<prefix>_message_level,
        defaulting to django.contrib.messages.WARNING when falsy.
        
        """
        return (
            self.get_denial_attr(prefix, "message_level") or
            messages.WARNING)
            
    def get_denial_message_tags(self, prefix):
        """
        A hook to customize resolution of the message tags used
        by deny().
        
        The default implementation returns self.<prefix>_message_tags,
        defaulting to the empty string when falsy.
        
        """
        return (
            self.get_denial_attr(prefix, "message_tags") or "")
    
    def get_denial_redirect_url(self, prefix):
        """
        A hook to customize resolution of the redirect URL used
        by deny().
        
        The default implementation returns self.<prefix>_redirect_url,
        defaulting to the resolved value for settings.LOGIN_URL when falsy.
        
        """
        redirect_url = self.get_denial_attr(prefix, "redirect_url")
        try:
            redirect_url = (
                redirect_url or urlresolvers.reverse(settings.LOGIN_URL))
        except urlresolvers.NoReverseMatch:
            redirect_url = settings.LOGIN_URL
        return redirect_url
    
    def get_denial_redirect_next_url(self, prefix):
        """
        A hook to customize resolution of the redirect's next URL used
        by deny().
        
        The default implementation returns self.<prefix>_redirect_next_url,
        defaulting to the request's fully-qualified URL when falsy.
        
        """
        return (
            self.get_denial_attr(prefix, "redirect_next_url") or
            self.request.build_absolute_uri())
    
    def get_denial_redirect_next_name(self, prefix):
        """
        A hook to customize resolution of the redirect next URL query parameter
        name used by deny().
        
        The default implementation returns self.<prefix>_redirect_next_name.
        
        """
        return self.get_denial_attr(prefix, "redirect_next_name")
    
    def get_denial_full_redirect_url(self, prefix):
        """
        A hook to customize building of the full redirect URL used
        by deny().
        
        The default implementation updates the query string for
        self.get_denial_redirect_url() to include a parameter named
        self.get_denial_redirect_next_name() with a value of
        self.get_denial_redirect_next_url(). The query string is updated
        only when the parameter name is truthy. Simplifies the query string
        value with respect to the redirect URL and request.
        
        """
        redirect_url = self.get_denial_redirect_url(prefix)
        next_name = self.get_denial_redirect_next_name(prefix)
        if next_name:
            return urlresolvers.update_query(
                redirect_url, {
                    next_name: urlresolvers.simplify_redirect(
                        self.get_denial_redirect_next_url(prefix),
                        redirect_url,
                        request=self.request)})
        return redirect_url
    
    # Hooks to modify the implementation of deny().
    def denial_raise_exception(self, prefix):
        """
        A hook to customize raising of an exception in deny().
        
        In the default implementation, when self.get_denial_raise() is
        truthy, raises self.get_denial_exception().
        
        """
        if self.get_denial_raise(prefix):
            raise self.get_denial_exception(prefix)
    
    def denial_enqueue_message(self, prefix):
        """
        A hook to customize message enqueuing in deny().
        
        The default implementation enqueues self.get_denial_messge() using
        the other self.get_denial_message_*() settings when the message
        is truthy.
        
        """
        message = self.get_denial_message(prefix)
        if message:
            messages.add_message(
                self.request,
                self.get_denial_message_level(prefix),
                message,
                extra_tags=self.get_denial_message_tags(prefix))
    
    def denial_respond(self, prefix):
        """
        A hook to customize the response upon in deny().
        
        The default implementation resolves the redirect URL with
        self.get_denial_full_redirect_url() and returns the
        resolved URL in a redirect response.
        
        """
        return http.HttpResponseRedirect(
            self.get_denial_full_redirect_url(prefix))
    
    def deny(self, prefix):
        """
        The handler that should be called upon access test failure with
        the prefix name for the behavior-controlling attributes.
        
        The default implementation optionally attempts to raise an exception
        with denial_raise_exception(). Then, it optionally enqueues a
        message with denial_enqueue_message(). Finally, it returns a
        redirect response which may include an optional next URL query string
        value with denial_respond().
        
        """
        self.denial_raise_exception(prefix)
        self.denial_enqueue_message(prefix)
        return self.denial_respond(prefix)



from __future__ import unicode_literals

from django.utils.text import compress_string

from daydreamer.tests.views import generic


class TestCase(generic.TestCase):
    """
    Common utilities for testing gzip view behaviors.
    
    """
    def compress(self, text):
        """
        Returns a compressed version of the given text, using the same
        function as Django's gzip decorator and middleware.
        
        """
        return compress_string(text)
    
    def unique_gzip(self):
        """
        Returns a unique string that satisifies the requirements for Django's
        gzip compression middleware.
        
        """
        # Content must have a length of at least 200, and it must be shorter
        # when gzipped.
        content = self.unique()
        while (
            len(content) < 200 or
            len(content) < len(self.compress(content))):
            content *= 2
        return content



from __future__ import unicode_literals

from daydreamer.tests.views import generic


class TestCase(generic.TestCase):
    """
    Common utilities for testing debug view behaviors.
    
    """
    def unique_variable(self):
        """
        Returns a unique variable name.
        
        """
        return "".join(("_", self.unique().replace("-", "_"),))



from __future__ import unicode_literals

import collections

from daydreamer.core import lang

from .. import testcases
from . import base


class TestCase(base.TestCase, testcases.TestCase):
    """
    A test case that provides a variety of general testing features
    for class-based views.
    
    Set the view_classes attribute to specify which view class(es) to inherit
    from for the test cases.
    
    """
    view_classes = None
    
    def view(self, **attrs):
        """
        Hardcodes self.view_classes as the first argument.
        
        """
        return super(TestCase, self).view(self.view_classes, **attrs)
    
    # Assertions.
    def assertViewBehavior(self, attrs=None, setup=None, **kwargs):
        """
        Sends an HTTP request to the view created from attrs with arguments
        and keyword arguments using the client's method.
        
        If the setup() callback is specified, its optional return value
        will be mixed into the view attributes.
        
        See daydreamer.test.views.testcases.TestCase.assertViewBehavior for
        documentation of the other arguments.
        
        """
        return super(TestCase, self).assertViewBehavior(
            self.view(
                **lang.updated(
                    attrs or {},
                    setup() or {}
                        if isinstance(setup, collections.Callable)
                        else {})),
            **kwargs)



from __future__ import unicode_literals

from django import http, template
from django.conf import settings

from daydreamer.views.behaviors import csrf

from . import base


class CsrfProtectTestCase(base.TestCase):
    """
    Tests for the CsrfProtect view behavior.
    
    CSRF middleware is turned off to enable edge case behavior.
    
    """
    view_classes = csrf.CsrfProtect
    
    def test_protect_post_no_cookie(self):
        """
        Check that POST requests without a cookie are CSRF protected.
        
        """
        self.assertViewBehavior(
            {"post": self.unique()},
            method="post",
            status_code=403)
    
    def test_protect_post_cookie(self):
        """
        Check that POST requests with a cookie are CSRF protected.
        
        """
        self.client.set_csrf_cookie()
        self.test_protect_post_no_cookie()
    
    def test_protect_put_no_cookie(self):
        """
        Checks that PUT requests without a cookie are CSRF protected.
        
        """
        self.assertViewBehavior(
            {"put": self.unique()},
            method="put",
            status_code=403)
    
    def test_protect_put_cookie(self):
        """
        Checks that PUT requests with a cookie are CSRF protected.
        
        """
        self.client.set_csrf_cookie()
        self.test_protect_put_no_cookie()
    
    def test_protect_patch_no_cookie(self):
        """
        Checks that PATCH requests without a cookie are CSRF protected.
        
        """
        self.assertViewBehavior(
            {"patch": self.unique()},
            method="patch",
            status_code=403)
    
    def test_protect_patch_cookie(self):
        """
        Checks that PATCH requests with a cookie are CSRF protected.
        
        """
        self.client.set_csrf_cookie()
        self.test_protect_patch_no_cookie()
    
    def test_protect_delete_no_cookie(self):
        """
        Checks that DELETE requests without a cookie are CSRF protected.
        
        """
        self.assertViewBehavior(
            {"delete": self.unique()},
            method="delete",
            status_code=403)
    
    def test_protect_delete_cookie(self):
        """
        Checks that DELETE requests with a cookie are CSRF protected.
        
        """
        self.client.set_csrf_cookie()
        self.test_protect_delete_no_cookie()
    
    def test_protect_precedence_no_cookie(self):
        """
        Checks that requests without a cookie are CSRF protected, taking
        precedence over the default HTTP method name protection.
        
        """
        self.assertViewBehavior(
            method="post",
            status_code=403)
    
    def test_protect_precedence_cookie(self):
        """
        Checks that requests with a cookie are CSRF protected, taking
        precedence over the default HTTP method name protection.
        
        """
        self.client.set_csrf_cookie()
        self.test_protect_precedence_no_cookie()
    
    def test_allow_post_data(self):
        """
        Check that POST requests are allowed with a CSRF cookie and appropriate
        POST data.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"post": content},
            method="post",
            data={"csrfmiddlewaretoken": self.client.get_csrf_cookie()},
            status_code=200,
            content=content)
    
    def test_allow_post_header(self):
        """
        Check that POST requests are allowed with a CSRF cookie and an
        appropriate header.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"post": content},
            method="post",
            headers={"HTTP_X_CSRFTOKEN": self.client.get_csrf_cookie()},
            status_code=200,
            content=content)
    
    def test_allow_post_disabled(self):
        """
        Check that POST requests are allowed when the view decorator
        is disabled.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"csrf_protect": False, "post": content},
            method="post",
            status_code=200,
            content=content)
    
    def test_allow_put_header(self):
        """
        Check that PUT requests are allowed with a CSRF cookie and an
        appropriate header.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"put": content},
            method="put",
            headers={"HTTP_X_CSRFTOKEN": self.client.get_csrf_cookie()},
            status_code=200,
            content=content)
    
    def test_allow_put_disabled(self):
        """
        Check that PUT requests are allowed when the view decorator
        is disabled.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"csrf_protect": False, "put": content},
            method="put",
            status_code=200,
            content=content)
    
    def test_allow_patch_header(self):
        """
        Check that PATCH requests are allowed with a CSRF cookie and an
        appropriate header.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"patch": content},
            method="patch",
            headers={"HTTP_X_CSRFTOKEN": self.client.get_csrf_cookie()},
            status_code=200,
            content=content)
    
    def test_allow_patch_disabled(self):
        """
        Check that PATCH requests are allowed when the view decorator
        is disabled.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"csrf_protect": False, "patch": content},
            method="patch",
            status_code=200,
            content=content)
    
    def test_allow_delete_header(self):
        """
        Check that DELETE requests are allowed with a CSRF cookie and an
        appropriate header.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"delete": content},
            method="delete",
            headers={"HTTP_X_CSRFTOKEN": self.client.get_csrf_cookie()},
            status_code=200,
            content=content)
    
    def test_allow_delete_disabled(self):
        """
        Check that DELETE requests are allowed when the view decorator
        is disabled.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"csrf_protect": False, "delete": content},
            method="delete",
            status_code=200,
            content=content)
    
    def test_allow_precedence_data(self):
        """
        Check that a POST request is allowed with a CSRF cookie and appropriate
        POST data and that the default HTTP method name protection
        takes precedence.
        
        """
        self.assertViewBehavior(
            method="post",
            data={"csrfmiddlewaretoken": self.client.get_csrf_cookie()},
            status_code=405)
    
    def test_allow_precedence_header(self):
        """
        Check that a POST request is allowed with a CSRF cookie and an
        appropriate header and that the default HTTP method name protection
        takes precedence.
        
        """
        self.assertViewBehavior(
            method="post",
            headers={"HTTP_X_CSRFTOKEN": self.client.get_csrf_cookie()},
            status_code=405)


class RequireCsrfTokenTestCase(base.TestCase):
    """
    Tests for the RequiresCsrfToken view behavior.
    
    CSRF middleware is turned off to enable edge case behavior.
    
    """
    view_classes = csrf.RequiresCsrfToken
    
    def test_csrf_token(self):
        """
        A csrf_token value should be added to a RequestContext.
        
        """
        content = self.unique()
        def get(self, request, *args, **kwargs):
            return http.HttpResponse(
                template.Template(content).render(
                    template.RequestContext(request, {})))
        self.assertViewBehavior(
            {"get": get},
            status_code=200,
            content=content,
            context_includes="csrf_token")
    
    def test_csrf_token_disabled(self):
        """
        A csrf_token value should be added to a RequestContext even when the
        view decorator is disabled, because of a hard-coded logic path
        in Django's RequestContext code.
        
        This shows that this view behavior is not very useful.
        
        """
        content = self.unique()
        def get(self, request, *args, **kwargs):
            return http.HttpResponse(
                template.Template(content).render(
                    template.RequestContext(request, {})))
        self.assertViewBehavior(
            {"requires_csrf_token": False, "get": get},
            status_code=200,
            content=content,
            context_includes="csrf_token")


class EnsureCsrfCookieTestCase(base.TestCase):
    """
    Tests for the EnsureCsrfCookie view behavior.
    
    CSRF middleware is turned off to enable edge case behavior.
    
    """
    view_classes = csrf.EnsureCsrfCookie
    
    def test_csrf_cookie(self):
        """
        Check that the CSRF cookie appears in the response.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content},
            status_code=200,
            content=content,
            cookies_include=settings.CSRF_COOKIE_NAME)
    
    def test_csrf_cookie_disabled(self):
        """
        Check that the CSRF cookie does not appear in the response.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"ensure_csrf_cookie": False, "get": content},
            status_code=200,
            content=content,
            cookies_exclude=settings.CSRF_COOKIE_NAME)


class CsrfExemptTestCase(base.TestCase):
    """
    Tests for the CsrfExempt view behavior.
    
    CSRF middleware is turned on to enable edge case behavior.
    
    """
    view_classes = csrf.CsrfExempt
    csrf_middleware_enabled = True
    
    def test_post_csrf_exempt(self):
        """
        Check that POST requests are CSRF exempt.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"post": content},
            method="post",
            status_code=200,
            content=content)
    
    def test_put_csrf_exempt(self):
        """
        Check that PUT requests are CSRF exempt.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"put": content},
            method="put",
            status_code=200,
            content=content)
    
    def test_patch_csrf_exempt(self):
        """
        Check that PATCH requests are CSRF exempt.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"patch": content},
            method="patch",
            status_code=200,
            content=content)
    
    def test_delete_csrf_exempt(self):
        """
        Check that DELETE requests are CSRF exempt.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"delete": content},
            method="delete",
            status_code=200,
            content=content)
    
    def test_post_csrf_exempt_disabled(self):
        """
        Check that POST requests are protected when the decorator
        is disabled.
        
        """
        self.assertViewBehavior(
            {"csrf_exempt": False, "post": self.unique()},
            method="post",
            status_code=403)
    
    def test_put_csrf_exempt_disabled(self):
        """
        Check that PUT requests are protected when the decorator
        is disabled.
        
        """
        self.assertViewBehavior(
            {"csrf_exempt": False, "put": self.unique()},
            method="put",
            status_code=403)
    
    def test_patch_csrf_exempt_disabled(self):
        """
        Check that PATCH requests are protected when the decorator
        is disabled.
        
        """
        self.assertViewBehavior(
            {"csrf_exempt": False, "patch": self.unique()},
            method="patch",
            status_code=403)
    
    def test_post_csrf_exempt_disabled(self):
        """
        Check that DELETE requests are protected when the decorator
        is disabled.
        
        """
        self.assertViewBehavior(
            {"csrf_exempt": False, "delete": self.unique()},
            method="delete",
            status_code=403)


class CsrfProtectCsrfExemptTestCase(CsrfExemptTestCase):
    """
    Tests for views inheriting from CsrfProtect followed by CsrfExempt.
    
    CSRF middleware is turned off to enable edge case behavior.
    
    This demonstrates the correct inheritance pattern to follow if you
    want easy control over CSRF protection in an environment with CSRF
    middleware disabled.
    
    """
    view_classes = (csrf.CsrfProtect, csrf.CsrfExempt,)
    csrf_middleware_enabled = False



from __future__ import unicode_literals

from daydreamer.core import lang

from . import context, cookies, headers, messages, redirects, requests


class TestCase(requests.TestCase, headers.TestCase, cookies.TestCase,
        context.TestCase, messages.TestCase, redirects.TestCase):
    """
    A test case that provides a variety of general testing features and an
    enhanced version of the base class' assertViewBehavior().
    
    """
    def assertViewBehavior(self, path, method="get",
            method_args=None, method_kwargs=None,
            request_includes=None, request_excludes=None, request_exact=None,
            headers_include=None, headers_exclude=None, headers_exact=None,
            cookies_include=None, cookies_exclude=None, cookies_exact=None,
            context_includes=None, context_excludes=None, context_exact=None,
            message=None, message_level=None, message_tags=None,
            message_limit=None,
            redirect_url=None, redirect_next_url=None,
            redirect_next_name=None, redirect_status_code=None,
            redirect_target_status_code=None,
            **kwargs):
        """
        Sends an HTTP request to the given path using the specified method.
        
        If method_args or method_kwargs is specified, their values will
        take precedence to allow for cooperative super() calls.
        
        If request_includes is specified, the request passed to the generated
        view must have the specified attributes after the view has been called.
        If exclude_response is specified, the request passed to the generated
        view must not have the specified attributes after the view has been
        called. If exact_response is specified, the request passed to the
        generated view must have the specified names and exact attribute values
        after the view has been called.
        
        If headers_include is specified, the response must contain the header
        name or names. If headers_exclude is specified, the response must not
        contain the header name or names. If headers_exact is specified, its
        names and exact values must be present in the response's headers.
        
        If cookies_include is specified, the response must contain the cookie
        name or names. If cookies_exclude is specified, the response must not
        contain the cookie name or names. If cookies_exact is specified, its
        names and exact values must be present in the response's cookies.
        
        If context_includes is specified, the response's context must contain
        the name or names. If context_excludes is specified, the response's
        context must not contain the name or names. If context_exact is
        specified, the names and exact values must be present in the response's
        context.
        
        If message is specified, checks that a message with the given message
        level and tags is present in the response's context, optionally
        checking that message limit count. Otherwise, checks that no messages
        are present in the response's context.
        
        If redirect_url is specified, checks that the response redirects to a
        URL which may have a query parameter specifying the next URL with the
        value of redirect_next_url and a name of redirect_next_name when
        the both are specified. The redirect status code and the target status
        will also be checked if specified.
        
        See daydreamer.test.base.TestCase.assertViewBehavior for documentation
        of the other arguments.
        
        """
        # Use the passed method arguments and keyword arguments or
        # generate them?
        if method_args is None or method_kwargs is None:
            method_args = (path,)
            method_kwargs = None
        
        # Set up the assertions.
        request_assertions = lang.updated(
            kwargs.pop("request_assertions", {}),
            dict(
                request_includes=request_includes,
                request_excludes=request_excludes,
                request_exact=request_exact),
            copy=True)
        
        response_assertions = lang.updated(
            kwargs.pop("response_assertions", {}),
            dict(
                headers_include=headers_include,
                headers_exclude=headers_exclude,
                headers_exact=headers_exact,
                cookies_include=cookies_include,
                cookies_exclude=cookies_exclude,
                cookies_exact=cookies_exact,
                context_includes=context_includes,
                context_excludes=context_excludes,
                context_exact=context_exact,
                message=message,
                message_level=message_level,
                message_tags=message_tags,
                message_limit=message_limit,
                redirect_url=redirect_url,
                redirect_next_url=redirect_next_url,
                redirect_next_name=redirect_next_name,
                redirect_status_code=redirect_status_code,
                redirect_target_status_code=redirect_target_status_code),
            copy=True)
        
        return super(TestCase, self).assertViewBehavior(
            method=method, method_args=method_args,
            method_kwargs=method_kwargs,
            request_assertions=request_assertions,
            response_assertions=response_assertions,
            **kwargs)



from __future__ import unicode_literals

import logging

from django import http
from django.core import exceptions
from django.views import generic

from daydreamer.core import lang, urlresolvers


__all__ = ("Core", "Null", "Allow", "Deny",)


class Core(generic.View):
    """
    Provides helpful features for generating common responses and
    reversing URLs.
    
    URL reversing is handled by daydreamer.core.urlresolvers.reverse().
    Because the request is available, it is easy to reverse fully-qualified
    URLs with the request's host domain and scheme. Simply pass qualified=True
    to self.reverse().
    
    Included responses are file attachments, redirects (301 and 302) and
    gone (410). Helpers that raise appropriate exceptions for not found (404),
    permission denied (403) and suspicious operation (400) are also provided.
    The not modified (304), method not allowed (405) and server error (500)
    responses are not provided, as they are handled automatically by other
    Django mechanisms.
    
    Even though the not_found(), permission_denied() and suspicious_operation()
    methods raise exceptions, it is recommended to return the responses from
    these calls for stylistic consistency, e.g.:
    
        # Do this:
        if resource.not_available():
            return self.not_found()
        
        # Not this:
        if resource.not_available():
            self.not_found()
    
    Since these methods raise exceptions, rather than return responses, take
    care to avoid catching their exceptions, or Django's special response
    exception handling machinery will be neutralized.
    
    """
    # The request logger.
    logger = logging.getLogger("django.request")
    
    # URL reversing.
    def reverse(self, viewname, qualified=False, scheme=None, **kwargs):
        """
        Reverse a URL from the given viewname and other optional arguments to
        django.contrib.urlresolvers.reverse(). If qualified is True, the result
        will include the URL scheme and host. If the scheme is also specified,
        it will take precedence over the current request's scheme.
        
        """
        return urlresolvers.reverse(viewname, **lang.updated(kwargs, (
            {"qualified": True, "scheme": scheme, "request": self.request}
                if qualified
                else {}),
            copy=True))
    
    # Responses.
    def attachment(self, data, content_type, filename):
        """
        Returns a file attachment response with the data as its payload
        and the specified content type (MIME type) and filename.
        
        """
        return lang.updated(
            http.HttpResponse(data, content_type=content_type), {
                "Content-Disposition":
                    "attachment; filename=\"{filename:s}\"".format(
                        filename=filename)})
    
    def redirect(self, viewname, permanent=False, **kwargs):
        """
        Returns a 301 or 302 redirect response with the reversed URL from the
        given viewname and other optional arguments to View.reverse().
        If permanent is True, the redirect will be a permanent 301 redirect.
        Otherwise, it will be a temporary 302 redirect.
        
        """
        return (
            http.HttpResponsePermanentRedirect
                if permanent
                else http.HttpResponseRedirect)(
                    self.reverse(viewname, **kwargs))
    
    def gone(self, debug_message=""):
        """
        Logs and returns a 410 gone response.
        
        """
        self.logger.info(
            "Gone ({method:s}: {path:s})".format(
                method=self.request.method, path=self.request.path),
            extra={"status_code": 410, "request": self.request})
        return http.HttpResponseGone(debug_message)
    
    def not_found(self, debug_message=""):
        """
        Raises an Http404 exception to trigger the 404 not found response
        machinery. Even though this raises an exception, it is recommended
        to return it for stylistic consistency:
        
            return self.not_found()
        
        """
        raise http.Http404(debug_message)
    
    def permission_denied(self, debug_message=""):
        """
        Raises a PermissionDenied exception to trigger the 403 forbidden
        response machinery. Even though this raises an exception, it is
        recommended to return it for stylistic consistency:
        
            return self.permission_denied()
        
        """
        raise exceptions.PermissionDenied(debug_message)
    
    def suspicious_operation(self, debug_message=""):
        """
        Raises a SuspiciousOperation exception to trigger the 400 bad request
        response machinery. Even though this raises an exception, it is
        recommended to return it for stylistic consistency:
        
            return self.suspicious_operation()
        
        """
        raise exceptions.SuspiciousOperation(debug_message)


class Null(Core):
    """
    The null view. Dispatches directly to the HTTP method denied handler.
    
    """
    def dispatch(self, request, *args, **kwargs):
        """
        The buck stops here. The request's method is denied.
        
        """
        return self.http_method_not_allowed(request, *args, **kwargs)


class Allow(Null):
    """
    The allow view. Selects a handler to allow the request or defers
    to super().
    
    """
    def get_allow_handler(self):
        """
        Either return a handler to allow the request or defer to super().
        
        """
        return None
    
    def dispatch(self, request, *args, **kwargs):
        """
        Call the allow handler or defer to super().
        
        """
        return (
            self.get_allow_handler() or
            super(Allow, self).dispatch)(
                request, *args, **kwargs)


class Deny(Null):
    """
    The denial view. Selects a handler to deny the request or defers
    to super().
    
    """
    def get_deny_handler(self):
        """
        Either return a handler to deny the request or defer to super().
        
        """
        return None
    
    def dispatch(self, request, *args, **kwargs):
        """
        Call the deny handler or defer to super().
        
        """
        return (
            self.get_deny_handler() or 
            super(Deny, self).dispatch)(
                request, *args, **kwargs)



from __future__ import unicode_literals

from django.views import generic

from .base import View


__all__ = ("DetailView",)


class DetailView(generic.DetailView, View):
    """
    Extends Django's DetailView class with features from
    daydreamer.views.generic.View.
    
    """
    pass



from __future__ import unicode_literals

import collections
import sys
import uuid

from django import test
from django.utils import encoding, six

from daydreamer.core import lang

from . import client


class TestCase(test.TestCase):
    """
    A test case that enhances Django's base test case with common testing
    utilities. Includes the assertBehavior(), which provides a flexible
    way to send an HTTP request and make assertions about the request
    and response objects.
    
    To turn on Django's CSRF framework during testing, set the
    enforce_csrf_checks attribute to True.
    
    """
    enforce_csrf_checks = False
    client_class = client.Client
    
    # Setup.
    def _pre_setup(self):
        """
        Replaces the default test client, providing a hook to specify whether
        to use Django's CSRF framework during testing.
        
        """
        super(TestCase, self)._pre_setup()
        self.client = self.client_class(
            enforce_csrf_checks=self.enforce_csrf_checks)
    
    # Utilities.
    def unique(self):
        """
        Returns a unique string (a UUID), useful for dummy values in tests.
        Helps to guarantee that functions or methods are not returning
        hardcoded values that coincidentally match a value used for
        a test.
        
        """
        return encoding.force_text(uuid.uuid4())
    
    # Assertions.
    def assertKeysIn(self, keys, data, message=None):
        """
        Asserts that the keys are present in the data.
        
        """
        for key in (
            (keys,)
                if isinstance(keys, six.string_types)
                else keys or ()):
            self.assertIn(
                key, data,
                message or "The key {key!r} was not found.".format(
                    key=key))
    
    def assertKeysNotIn(self, keys, data, message=None):
        """
        Asserts that the keys are not present in the data.
        
        """
        for key in (
            (keys,)
                if isinstance(keys, six.string_types)
                else keys or ()):
            self.assertNotIn(
                key, data,
                message or "The key {key!r} was found.")
    
    def assertItemsExact(self, data, items, message=None):
        """
        Asserts that the keys specified in items are present in data and that
        their values are equal.
        
        """
        for key, value in six.iteritems(
            items
                if isinstance(items, collections.Mapping)
                else dict(items or {})):
            self.assertIn(
                key, data,
                "The key {key!r} was not found.".format(key=key))
            self.assertEqual(
                data[key], value, (
                    message or
                    "The {actual!r} value for the key {key!r} was not equal "
                    "to {value!r}.".format(
                        actual=data[key], key=key, value=value)))
    
    def assertAttributesIn(self, attrs, obj, message=None):
        """
        Asserts that the attributes are present in the object.
        
        """
        for attr in (
            (attrs,)
                if isinstance(attrs, six.string_types)
                else attrs or ()):
            self.assertTrue(
                hasattr(obj, attr),
                message or "The attribute {attr!r} was not found.".format(
                    attr=attr))
    
    def assertAttributesNotIn(self, attrs, obj, message=None):
        """
        Asserts that the attributes are not present in the object.
        
        """
        for attr in (
            (attrs,)
                if isinstance(attrs, six.string_types)
                else attrs or ()):
            self.assertFalse(
                hasattr(obj, attr),
                message or "The attribute {attr!r} was found.".format(
                    attr=attr))
    
    def assertAttributesExact(self, obj, items, message=None):
        """
        Asserts that the attributes from items are present in the object and
        that their values are equal.
        
        """
        for attr, value in six.iteritems(
            items
                if isinstance(items, collections.Mapping)
                else dict(items or {})):
            self.assertTrue(
                hasattr(obj, attr),
                "The atrribute {attr!r} was not found".format(
                    attr=attr))
            self.assertEqual(
                getattr(obj, attr), value, (
                    message or
                    "The {actual!r} value for the attribute {attr!r} was not "
                    "equal to {value!r}.".format(
                        actual=getattr(obj, attr), attr=attr, value=value)))
    
    def assertViewBehavior(self, method="get",
            method_args=None, method_kwargs=None,
            data=None, follow=False, headers=None,
            setup=None, repeat=None, exception=None,
            status_code=None, content=None,
            method_assertions=None,
            request_assertions=None,
            response_assertions=None):
        """
        Sends an HTTP request for the specified method including optional
        data or headers and optionally following redirects. Makes a series of
        assertions about the underlying view's behavior.
        
        The method along with its method_args and method_kwargs will be used to
        send a request with the test case's client.
        
        If data is specified, it should be a dictionary for a "get", "post"
        or "head" request and a string otherwise. When specified, its value
        will be added to method_kwargs under the "data" key.
        
        If follow is True, redirects will be followed recursively. Its value
        will be added to method_kwargs under the "follow" key.
        
        If headers is specified, it should be a dictionary containing header
        values for the request, which will be mixed into method_kwargs.
        
        If the setup() callback is specified, it will be called before the
        request is sent.
        
        If repeat is specified, it should indicate the number of times to
        repeat the request. Each call will be wrapped in an exception handler
        until the last call, which will be processed by the assertions.
        
        If exception is specified, the request should raise the exception,
        and no other assertions will be performed.
        
        If status code is specified, the response's status code must match.
        
        If content is specified, the response's content must match.
        
        If any of method_assertions, request_assertions or response_assesrtions
        is specified, it should be a dictionary of values to pass to the
        corresponding get_<type>_assertions() methods.
        
        """
        # Sanity check and normalize the arguments.
        method_args = method_args or ()
        method_kwargs = (
            method_kwargs
                if isinstance(method_kwargs, collections.Mapping)
                else dict(method_kwargs or {}))
        if data is not None:
            if method in ("get", "post", "head",):
                if not isinstance(data, collections.Mapping):
                    raise ValueError(
                        'The data must be a dictionary for "get", "post" or '
                        '"head" requests.')
            else:
                if not isinstance(data, six.string_types):
                    raise ValueError(
                        "The data must be a string for request methods other "
                        'than "get", "post" or "head".')
        else:
            data = {} if method in ("get", "post", "head",) else ""
        headers = (
            headers
                if isinstance(headers, collections.Mapping)
                else dict(headers or {}))
        repeat = repeat or 0
        method_assertions = (
            method_assertions
                if isinstance(method_assertions, collections.Mapping)
                else dict(method_assertions or {}))
        request_assertions = (
            request_assertions
                if isinstance(request_assertions, collections.Mapping)
                else dict(request_assertions or {}))
        response_assertions = (
            response_assertions
                if isinstance(response_assertions, collections.Mapping)
                else dict(response_assertions or {}))
        
        # Mix data and headers into the method keyword arguments.
        method_kwargs = lang.updated(
            method_kwargs,
            dict(
                ((("data", data),) if data else ()) +
                (("follow", follow),) +
                tuple(six.iteritems(headers))),
            copy=True)
        
        # Burn through the repetitions?
        while repeat > 1:
            try:
                getattr(self.client, method)(
                    *method_args, **method_kwargs)
            except SystemExit:
                six.reraise(*sys.exc_info())
            except:
                pass
            repeat -= 1
        
        # Do we expect an exception or a normal response?
        if exception:
            with self.assertRaises(
                exception.__class__
                    if isinstance(exception, Exception)
                    else exception):
                getattr(self.client, method)(
                    *method_args, **method_kwargs)
        else:
            response = getattr(self.client, method)(
                *method_args, **method_kwargs)
            request = self.client.handler.last_request
            if status_code is not None:
                self.assertEqual(response.status_code, status_code)
            if content is not None:
                self.assertEqual(response.content, content)
            for assertion in self.create_method_assertions(
                **method_assertions):
                assertion(*method_args or ())
            for assertion in self.create_request_assertions(
                **request_assertions):
                assertion(request)
            for assertion in self.create_response_assertions(
                **response_assertions):
                assertion(response)
    
    def create_method_assertions(self):
        """
        Subclasses should accept arbitrary keyword arguments and pass any
        unused keywords to super(). That way, this base implementation will
        raise an exception if keywords have gone unused.
        
        """
        return ()
    
    def create_request_assertions(self):
        """
        A hook for creating request assertions. Must return a tuple of
        callbacks that accept the request that was generated to make
        the request.
        
        Subclasses should accept arbitrary keyword arguments and pass any
        unused keywords to super(). That way, this base implementation will
        raise an exception if keywords have gone unused.
        
        """
        return ()
    
    def create_response_assertions(self):
        """
        A hook for creating response assertions. Must return a tuple of
        callbacks that accept the response that was generated by making
        the request.
        
        Subclasses should accept arbitrary keyword arguments and pass any
        unused keywords to super(). That way, this base implementation will
        raise an exception if keywords have gone unused.
        
        """
        return ()



from __future__ import unicode_literals

from django.utils.decorators import classonlymethod
from django.views.decorators import cache

from .. import core


__all__ = ("CachePage", "CacheControl", "NeverCache",)


class CachePage(core.http.HttpMethodAllow):
    """
    A view behavior that caches the response during the allow phase of
    the dispatch.
    
    See the django.views.decorators.cache.cache_page() decorator for details.
    
    """
    cache_page = True
    cache_page_timeout = None
    cache_page_cache = None
    cache_page_key_prefix = None
    
    def get_cache_page(self):
        """
        A hook to override the cache_page value.
        
        """
        return self.cache_page
    
    def get_cache_page_timeout(self):
        """
        A hook to override the cache_page_timeout value.
        
        """
        return self.cache_page_timeout
    
    def get_cache_page_cache(self):
        """
        A hook to override the cache_page_cache value.
        
        """
        return self.cache_page_cache
    
    def get_cache_page_key_prefix(self):
        """
        A hook to override the cache_page_key_prefix value.
        
        """
        return self.cache_page_key_prefix
    
    def get_allow_handler(self):
        """
        When cache_page is truthy, wraps the base handler in the
        django.views.decorators.cache.cache_page() decorator.
        
        """
        allow = super(CachePage, self).get_allow_handler()
        if self.get_cache_page():
            allow = cache.cache_page(
                self.get_cache_page_timeout(),
                cache=self.get_cache_page_cache(),
                key_prefix=self.get_cache_page_key_prefix())(allow)
        return allow


class CacheControl(core.http.HttpMethodAllow):
    """
    A view behavior that adds cache control headers to the response during
    the allow phase of the dispatch.
    
    Set the cache_control attribute to a falsy value to disable the behavior.
    
    Set the optional attributes to values other than None to use them. Rather
    having both a public and a private attribute, set public to True or False
    to set either the public or private header value, respectively.
    
    See the django.views.decorators.cache.cache_control() decorator
    for details.
    
    """
    cache_control = True
    cache_control_public = None
    cache_control_no_cache = None
    cache_control_no_transform = None
    cache_control_must_revalidate = None
    cache_control_proxy_revalidate = None
    cache_control_max_age = None
    cache_control_s_maxage = None
    
    def get_cache_control(self):
        """
        A hook to override the cache_control value.
        
        """
        return self.cache_control
    
    def get_cache_control_public(self):
        """
        A hook to override the cache_control_public value.
        
        """
        return self.cache_control_public
    
    def get_cache_control_no_cache(self):
        """
        A hook to override the cache_control_no_cache value.
        
        """
        return self.cache_control_no_cache
    
    def get_cache_control_no_transform(self):
        """
        A hook to override the cache_control_no_transform value.
        
        """
        return self.cache_control_no_transform
    
    def get_cache_control_must_revalidate(self):
        """
        A hook to override the cache_control_must_revalidate value.
        
        """
        return self.cache_control_must_revalidate
    
    def get_cache_control_proxy_revalidate(self):
        """
        A hook to override the cache_control_proxy_revalidate value.
        
        """
        return self.cache_control_proxy_revalidate
    
    def get_cache_control_max_age(self):
        """
        A hook to override the cache_control_max_age value.
        
        """
        return self.cache_control_max_age
    
    def get_cache_control_s_maxage(self):
        """
        A hook to override the cache_control_s_maxage value.
        
        """
        return self.cache_control_s_maxage
    
    def get_allow_handler(self):
        """
        When cache_control is truthy and at least one cache_control_* value
        is set, wraps the base handler in the
        django.views.decorated.cache.cache_control() decorator.
        
        """
        allow = super(CacheControl, self).get_allow_handler()
        if self.get_cache_control():
            public = self.get_cache_control_public()
            no_cache = self.get_cache_control_no_cache()
            no_transform = self.get_cache_control_no_transform()
            must_revalidate = self.get_cache_control_must_revalidate()
            proxy_revalidate = self.get_cache_control_proxy_revalidate()
            max_age = self.get_cache_control_max_age()
            s_maxage = self.get_cache_control_s_maxage()
            controls = dict(
                ((("public", True,),)
                    if public
                    else ()
                        if public is None
                        else (("private", True,),)) +
                ((("no_cache", True,),)
                    if no_cache
                    else ()) +
                ((("no_transform", True,),)
                    if no_transform
                    else ()) +
                ((("must_revalidate", True,),)
                    if must_revalidate
                    else ()) +
                ((("proxy_revalidate", True,),)
                    if proxy_revalidate
                    else ()) +
                ((("max_age", max_age,),)
                    if max_age
                    else ()) +
                ((("s_maxage", s_maxage,),)
                    if s_maxage
                    else ()))
            if controls:
                allow = cache.cache_control(**controls)(allow)
        return allow


class NeverCache(core.http.HttpMethodAllow):
    """
    A view behavior that sets up cache control headings to never cache during
    the allow phase of the dispatch.
    
    Set the never_cache attribute to falsy value to disable the behavior.
    
    See the django.views.decorators.cache.never_cache() decorator for details.
    
    """
    never_cache = True
    
    def get_never_cache(self):
        """
        A hook to override the never_cache value.
        
        """
        return self.never_cache
    
    def get_allow_handler(self):
        """
        When never_cache is truthy, wraps the base handler in
        django.views.decorators.cache.never_cache().
        
        """
        allow = super(NeverCache, self).get_allow_handler()
        if self.get_never_cache():
            allow = cache.never_cache(allow)
        return allow



from __future__ import unicode_literals

from django.views import generic

from .base import View


__all__ = (
    "ArchiveIndexView", "YearArchiveView", "MonthArchiveView",
    "WeekArchiveView", "DayArchiveView", "TodayArchiveView", "DateDetailView",)


class ArchiveIndexView(generic.ArchiveIndexView, View):
    """
    Extends Django's ArchiveIndexView class with features from
    daydreamer.views.generic.View.
    
    """
    pass


class YearArchiveView(generic.YearArchiveView, View):
    """
    Extends Django's YearArchiveView class with features from
    daydreamer.views.generic.View.
    
    """
    pass


class MonthArchiveView(generic.MonthArchiveView, View):
    """
    Extends Django's MonthArchiveView class with features from
    daydreamer.views.generic.View.
    
    """
    pass


class WeekArchiveView(generic.WeekArchiveView, View):
    """
    Extends Django's WeekArchiveView class with features from
    daydreamer.views.generic.View.
    
    """
    pass


class DayArchiveView(generic.DayArchiveView, View):
    """
    Extends Django's DayArchiveView class with features from
    daydreamer.views.generic.View.
    
    """
    pass


class TodayArchiveView(generic.TodayArchiveView, View):
    """
    Extends Django's TodayArchiveView class with features from
    daydreamer.views.generic.View.
    
    """
    pass


class DateDetailView(generic.DateDetailView, View):
    """
    Extends Django's DateDetailView class with features from
    daydreamer.views.generic.View.
    
    """
    pass



from . import base, behaviors, http
from .base import Core, Null, Allow, Deny
from .behaviors import Denial
from .http import HttpMethodAllow, HttpMethodDeny


__all__ = (
    "Core", "Null", "Allow", "Deny"
    "Denial",
    "HttpMethodAllow", "HttpMethodDeny",)



from __future__ import unicode_literals

from daydreamer.tests.views.core import behaviors


class TestCase(behaviors.TestCase):
    """
    Common utilities for testing authentication view behaviors.
    
    """
    pass



from __future__ import unicode_literals

import functools
import urlparse

from django.http import request

from daydreamer.core import lang, urlresolvers

from . import base


class TestCase(base.TestCase):
    """
    A test case for making assertions about redirects.
    
    """
    def assertRedirects(self, response, expected_url, query=None,
            status_code=302, target_status_code=None, host=None,
            msg_prefix=""):
        """
        Asserts that the response was redirected as expected. If provided,
        the query dictionary is added to the expected URL's query string. If
        the target status code is not specified, no assertion of its value
        will be performed.
        
        """
        # If unspecified, make sure the target status code will match the
        # base implementation's expected code.
        if target_status_code is None:
            if getattr(response, "redirect_chain", None):
                target_status_code = response.status_code
            else:
                parts = urlparse.urlparse(response.url)
                target_status_code = response.client.get(
                    parts.path, request.QueryDict(parts.query)).status_code
        
        return super(TestCase, self).assertRedirects(
            response, urlresolvers.update_query(expected_url, query),
            status_code=status_code, target_status_code=target_status_code,
            host=host, msg_prefix=msg_prefix)
    
    def create_redirect_assertions(self, expected_url=None, query=None,
            status_code=302, target_status_code=None):
        """
        Returns a tuple of redirect assertion callbacks for the
        given arguments.
        
        """
        return (
            (functools.partial(self.assertRedirects,
                expected_url=expected_url, query=query,
                status_code=status_code,
                target_status_code=target_status_code),)
                if expected_url is not None
                else ())
    
    def create_response_assertions(self, redirect_url=None,
            redirect_next_url=None, redirect_next_name=None,
            redirect_status_code=None, redirect_target_status_code=None,
            **kwargs):
        """
        Adds redirect assertions to the response assertions.
        
        """
        return (
            self.create_redirect_assertions(
                expected_url=redirect_url,
                query=(
                    {redirect_next_name: redirect_next_url}
                        if redirect_next_name
                        else None),
                status_code=(
                    redirect_status_code
                        if redirect_status_code is not None
                        else 302),
                target_status_code=redirect_target_status_code) +
            super(TestCase, self).create_response_assertions(**kwargs))



from __future__ import unicode_literals

import weakref

from daydreamer.core import lang

from .. import client
from . import handler


class ClientBaseImplementation(object):
    """
    A context manager that defers to the Client instance's base handler
    and methods.
    
    """
    deferred_methods = (
        "get", "post", "head", "options", "put", "patch", "delete",)
    
    def __init__(self, client, base_handler):
        self.client = weakref.ref(client)
        self.base_handler = base_handler
        self.handler = client.handler
        
        for method in self.deferred_methods:
            setattr(self, method, getattr(client, method))
    
    def __enter__(self):
        client = self.client()
        if client:
            client.handler = self.base_handler
            for method in self.deferred_methods:
                setattr(client, method, getattr(super(Client, client), method))
        return client
    
    def __exit__(self, exc_type, exc_value, traceback):
        client = self.client()
        if client:
            client.handler = self.handler
            for method in self.deferred_methods:
                setattr(client, method, getattr(self, method))


class Client(client.Client):
    """
    A test client that uses a customized request handler to directly test 
    a specified view with arguments and keyword arguments.
    
    """
    def __init__(self, enforce_csrf_checks=False, **defaults):
        """
        Replace the default handler with the customized request handler
        and set up a context manager for deffering to the base's handler
        and methods when desired.
        
        """
        super(Client, self).__init__(
            enforce_csrf_checks=enforce_csrf_checks, **defaults)
        
        # Replace the handler and set up the context manager.
        base_handler = self.handler
        self.handler = handler.ClientHandler(enforce_csrf_checks)
        self.base_implementation = ClientBaseImplementation(self, base_handler)
    
    def _view_request(self, view, view_args, view_kwargs):
        return {
            "django.view": view,
            "django.view_args": view_args or (),
            "django.view_kwargs": view_kwargs or {}}
    
    def get(self, view, view_args=None, view_kwargs=None, path="/", data={},
            follow=False, **extra):
        """
        Runs the given view with a GET request.
        
        """
        extra.pop("follow", None)
        return super(Client, self).get(path, data=data, follow=follow,
            **lang.updated(
                extra, self._view_request(view, view_args, view_kwargs)))
    
    def post(self, view, view_args=None, view_kwargs=None, path="/", data={},
            content_type=client.MULTIPART_CONTENT, follow=False, **extra):
        """
        Runs the given view with a POST request.
        
        """
        return super(Client, self).post(path, data=data,
            content_type=content_type, follow=follow,
            **lang.updated(
                extra, self._view_request(view, view_args, view_kwargs)))
    
    def head(self, view, view_args=None, view_kwargs=None, path="/", data={},
            follow=False, **extra):
        """
        Runs the given view with a HEAD request.
        
        """
        return super(Client, self).head(path, data=data, follow=follow,
            **lang.updated(
                extra, self._view_request(view, view_args, view_kwargs)))
    
    def options(self, view, view_args=None, view_kwargs=None, path="/", data="",
            content_type="application/octet-stream", follow=False, **extra):
        """
        Runs the given view with an OPTIONS request.
        
        """
        return super(Client, self).options(path, data=data,
            content_type=content_type, follow=follow,
            **lang.updated(
                extra, self._view_request(view, view_args, view_kwargs)))
    
    def put(self, view, view_args=None, view_kwargs=None, path="/", data="",
            content_type="application/octet-stream", follow=False, **extra):
        """
        Runs the given view with a PUT request.
        
        """
        return super(Client, self).put(path, data=data,
            content_type=content_type, follow=follow,
            **lang.updated(
                extra, self._view_request(view, view_args, view_kwargs)))
    
    def patch(self, view, view_args=None, view_kwargs=None, path="/", data="",
            content_type="application/octet-stream", follow=False, **extra):
        """
        Runs the given view with a PATCH request.
        
        """
        extra.pop("follow", None)
        return super(Client, self).patch(path, data=data,
            content_type=content_type, follow=follow,
            **lang.updated(
                extra, self._view_request(view, view_args, view_kwargs)))
    
    def delete(self, view, view_args=None, view_kwargs=None, path="/", data="",
            content_type="application/octet-stream", follow=False, **extra):
        """
        Runs the given view with a DELETE request.
        
        """
        return super(Client, self).delete(path, data=data,
            content_type=content_type, follow=follow,
            **lang.updated(
                extra, self._view_request(view, view_args, view_kwargs)))
    
    def _handle_redirects(self, *args, **kwargs):
        """
        Patches redirect handling to ensure that the base class' get() method
        and the original request handler are used.
        
        """
        with self.base_implementation:
            return super(Client, self)._handle_redirects(*args, **kwargs)



from __future__ import unicode_literals

from daydreamer.views.behaviors import clickjacking

from . import base


class XFrameOptionsDenyTestCase(base.TestCase):
    """
    Tests for the XFrameOptionsDeny view behavior.
    
    Clickjacking middleware is turned off to enable edge case behavior.
    
    """
    view_classes = clickjacking.XFrameOptionsDeny
    
    def test_deny_headers(self):
        """
        Check that the DENY X-Frame-Options header is set on the response.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content},
            status_code=200,
            content=content,
            headers_exact={"X-Frame-Options": "DENY"})
    
    def test_deny_disabled(self):
        """
        Check that no X-Frame-Options header is set on the response when the
        view behavior is disabled.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"xframe_options_deny": False, "get": content},
            status_code=200,
            content=content,
            headers_exclude="X-Frame-Options")
    
    def test_deny_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        and that the DENY X-Frame-Options header is set on the response.
        
        """
        self.assertViewBehavior(
            status_code=405,
            headers_exact={"X-Frame-Options": "DENY"})
    
    def test_deny_disabled_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        and that no X-Frame-Options header is set on the response when the
        view behavior is disabled.
        
        """
        self.assertViewBehavior(
            {"xframe_options_deny": False},
            status_code=405,
            headers_exclude="X-Frame-Options")


class XFrameOptionsSameOriginTestCase(base.TestCase):
    """
    Tests for the XFrameOptionsSameOrigin view behavior.
    
    Clickjacking middleware is turned off to enable edge case behavior.
    
    """
    view_classes = clickjacking.XFrameOptionsSameOrigin
    
    def test_same_origin_headers(self):
        """
        Check that the SAMEORIGIN X-Frame-Options header is set on
        the response.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content},
            status_code=200,
            content=content,
            headers_exact={"X-Frame-Options": "SAMEORIGIN"})
    
    def test_same_origin_disabled(self):
        """
        Check that no X-Frame-Options header is set on the response when the
        view behavior is disabled.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"xframe_options_same_origin": False, "get": content},
            status_code=200,
            content=content,
            headers_exclude="X-Frame-Options")
    
    def test_same_origin_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        and that the SAMEORIGIN X-Frame-Options header is set on the response.
        
        """
        self.assertViewBehavior(
            status_code=405,
            headers_exact={"X-Frame-Options": "SAMEORIGIN"})
    
    def test_same_origin_disabled_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        and that no X-Frame-Options header is set on the response when the
        view behavior is disabled.
        
        """
        self.assertViewBehavior(
            {"xframe_options_same_origin": False},
            status_code=405,
            headers_exclude="X-Frame-Options")


class XFrameOptionsExemptTestCase(base.TestCase):
    """
    Tests for the XFrameOptionsExempt view behavior.
    
    Clickjacking middleware is turned on to enable edge case behavior.
    
    """
    view_classes = clickjacking.XFrameOptionsExempt
    clickjacking_middleware_enabled = True
    
    def test_exempt(self):
        """
        Check that no X-Frame-Options header is set on the response.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content},
            status_code=200,
            content=content,
            headers_exclude="X-Frame-Options")
    
    def test_exempt_disabled(self):
        """
        Check that an X-Frame-Options header is set on the response when the
        view behavior is disabled.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"xframe_options_exempt": False, "get": content},
            status_code=200,
            content=content,
            headers_include="X-Frame-Options")
    
    def test_exempt_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        and that no X-Frame-Options header is set on the response.
        
        """
        self.assertViewBehavior(
            status_code=405,
            headers_exclude="X-Frame-Options")
    
    def test_exempt_disabled_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        and that an X-Frame-Options header is set on the response when the
        view behavior is disabled.
        
        """
        self.assertViewBehavior(
            {"xframe_options_exempt": False},
            status_code=405,
            headers_include="X-Frame-Options")



from __future__ import unicode_literals

from django.test import client

from . import handler


MULTIPART_CONTENT = client.MULTIPART_CONTENT


class Client(client.Client):
    """
    A custom test client that uses a request handler that provides
    a way to retrive the request object used for the last HTTP
    request sent by the client.
    
    A custom test client that provides a way to retrieve the request
    object used for the last HTTP request that was sent by the client.
    
    """
    def __init__(self, enforce_csrf_checks=False, **defaults):
        """
        Replace the default handler with the customized request handler.
        
        """
        super(Client, self).__init__(
            enforce_csrf_checks=enforce_csrf_checks, **defaults)
        self.handler = handler.ClientHandler(enforce_csrf_checks)



from __future__ import unicode_literals

import collections

from django.views.decorators import http

from .. import core


__all__ = ("RequireGET", "RequirePOST", "RequireSafe", "Condition",)


class RequireGET(core.http.HttpMethodDeny):
    """
    A view behavior that requires an HTTP GET request.
    
    """
    http_method_names = ("get",)


class RequirePOST(core.http.HttpMethodDeny):
    """
    A view behavior that requires an HTTP POST request.
    
    """
    http_method_names = ("post",)


class RequireSafe(core.http.HttpMethodDeny):
    """
    A view behavior that requires an HTTP GET or HEAD request.
    
    """
    http_method_names = ("get", "head",)


class Condition(core.http.HttpMethodAllow):
    """
    A view behavior that provides conditional retrieval or change
    notification during the allow phase of the dispatch.
    
    Define a condition_etag() method to compute the ETag for the requested
    resource and/or define a condition_last_modified() method to compute the
    last modified datetime for the requestd resource. Both methods should
    take the request, arguments and keyword arguments from the URL dispatcher,
    i.e. (self, request, *args, **kwargs). If neither method is defined,
    the behavior will be disabled.
    
    See the django.views.decorators.http.condition() view decorator
    for details.
    
    """
    condition_etag = None
    condition_last_modified = None
    
    def get_allow_handler(self):
        """
        If either of condition_etag or condition_last_modified is defined, wrap
        the base allow handler in the condition() decorator.
        
        """
        allow = super(Condition, self).get_allow_handler()
        if (isinstance(self.condition_etag, collections.Callable) or
            isinstance(self.condition_last_modified, collections.Callable)):
            allow = http.condition(
                etag_func=self.condition_etag,
                last_modified_func=self.condition_last_modified)(allow)
        return allow



from __future__ import unicode_literals

from daydreamer.views import generic
from daydreamer.views.behaviors import vary

from . import base


class VaryOnHeadersTestCase(base.TestCase):
    """
    Tests for the VaryOnHeaders view behavior.
    
    """
    view_classes = (vary.VaryOnHeaders, generic.View,)
    
    def test_vary_on_headers_single(self):
        """
        Check that a single Vary header value is set on the response.
        
        """
        content = self.unique()
        vary = self.unique()
        self.assertViewBehavior(
            {"vary_on_headers": vary, "get": content},
            status_code=200,
            content=content,
            headers_exact={"Vary": vary})
    
    def test_vary_on_headers_multiple(self):
        """
        Check that multiple Vary header values are set on the response.
        
        """
        content = self.unique()
        vary = (self.unique(), self.unique(),)
        self.assertViewBehavior(
            {"vary_on_headers": vary, "get": content},
            status_code=200,
            content=content,
            headers_exact={"Vary": ", ".join(vary)})
    
    def test_vary_on_headers_disabled(self):
        """
        Check that the behavior is disabled when no Vary header values
        are specified.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content},
            status_code=200,
            content=content,
            headers_exclude="Vary")
    
    def test_vary_on_headers_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        and that no Vary headers are set on the response.
        
        """
        vary = self.unique()
        self.assertViewBehavior(
            {"vary_on_headers": vary},
            status_code=405,
            headers_exclude="Vary")


class VaryOnCookieTestCase(base.TestCase):
    """
    Tests for the VaryOnCookie view behavior.
    
    """
    view_classes = (vary.VaryOnCookie, generic.View,)
    
    def test_vary_on_cookie(self):
        """
        Check that a Vary header value for cookies is set on the response.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content},
            status_code=200,
            content=content,
            headers_exact={"Vary": "Cookie"})
    
    def test_vary_on_cookie_disabled(self):
        """
        Check that the behavior is disabled when specified.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"vary_on_cookie": False, "get": content},
            status_code=200,
            content=content,
            headers_exclude="Vary")
    
    def test_vary_on_cookie_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        and that no Vary headers are set on the response.
        
        """
        vary = self.unique()
        self.assertViewBehavior(
            status_code=405,
            headers_exclude="Vary")
